#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
THOR detects differential peaks in multiple ChIP-seq profiles associated
with two distinct biological conditions.

Copyright (C) 2014-2016 Manuel Allhoff (allhoff@aices.rwth-aachen.de)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

@author: Manuel Allhoff
"""

from __future__ import print_function
# from optparse import OptionParser
import sys
from rgt.ExperimentalMatrix import ExperimentalMatrix
from rgt.CoverageSet import CoverageSet
import numpy as np
import os
from rgt.GenomicRegionSet import GenomicRegionSet
from copy import deepcopy

def get_experimental_matrix(bams, bed):
    """Load artificially experimental matrix. Only genes in BED file are needed."""
    m = ExperimentalMatrix()
    
    m.fields = ['name', 'type', 'file']
    m.fieldsDict = {}
    
    names = []
    for bam in bams:
        n, _ = os.path.splitext(os.path.basename(bam))
        m.files[n] = bam
        names.append(n) 
    m.names = np.array(['housekeep'] + names)
    m.types = np.array(['regions'] + ['reads']*len(names))
    g = GenomicRegionSet('RegionSet')
    g.read_bed(bed)
    m.objectsDict['housekeep'] = g
    
    return m

def get_factor_matrix(d, colnames, folder, samples, verbose, report):
    """Give matrix describing factors between genes. Idealy, factors in a column should be approx. the same."""
    res = []
    
    if report:
        f_gene = open(folder + '/pics/data/gene.data', 'w')
        f_sample = open(folder + 'pics/data/sample.data', 'w')
    
    original_f = get_factors(d)

    if d.shape[0] > 1 and d.shape[1] > 1:
        print("-gene (column) wise evaluation", file=sys.stderr)
        for i in range(d.shape[1]):
            data = deepcopy(d)
            data = np.delete(data, i, 1) #remove gene i
            f = get_factors(data)
            assert len(f) == len(original_f)
            res = sum(map(lambda x: (x[0]-x[1])**2, zip(original_f, f))) / float(len(f))
            
            if report:
                print(colnames[i], res, file=f_gene)
            
            if verbose:
                print(colnames[i], res, file=sys.stderr)
        
        if verbose:
            print("", file=sys.stderr)
        
            print("-sample (row) wise evaluation", file=sys.stderr)
        for i in range(d.shape[0]):
            data = deepcopy(d)
            data = np.delete(data, i, 0) #remove sample i
            f = get_factors(data)
            assert len(f) == len(original_f) - 1
            tmp = deepcopy(original_f)
            del tmp[i]
            assert len(f) == len(tmp)
            res = sum(map(lambda x: (x[0]-x[1])**2, zip(tmp, f))) / float(len(f))
            
            if report:
                print(samples[i], res, file=f_sample)
            if verbose:
                print(samples[i], res, file=sys.stderr)
        if verbose:
            print("", file=sys.stderr)
    
    if report:
        f_gene.close()
        f_sample.close()
     
def output_R_file(name, res, colnames):
    """"Write R code to file to check whether genes give same signal among the samples"""
    f = open(name + 'norm.R', 'w')
    #output for R
    #everthing in one vector
    
    #if res.shape[1] > 0:
    l = reduce(lambda x, y: x+y, [map(lambda x: str(x), list(np.array(res[:,i]).reshape(-1,))) for i in range(res.shape[1])])
    #else:
    #    l = list(np.array(res.reshape(-1,)))
    
    print('d = c(', ', '.join(l), ')', sep='', file=f)
    print('d = matrix(d, %s)' %res.shape[0], file=f)
    print('names = c("', '", "'.join(colnames), '")', sep='', file=f)
    print('par(mar=c(15,5,5,5))', file=f)
    print('barplot(d, beside = TRUE, names.arg = names, las=2, main="Housekeeping Genes Ratio", ylab="Signal")', file=f)
    


def get_factors(data):
    #normalize: increase values to highest value
    d = deepcopy(data)  
    colmax = np.mean(d, axis=0)
    for i in range(d.shape[0]):
        for j in range(d.shape[1]):
            d[i,j] = colmax[:,j][0,0]/d[i,j]
    
    return list(np.array(np.mean(d, axis=1)).reshape(-1))

def norm_gene_level(bams, bed, name, verbose, folder, report):
    """Normalize bam files on a gene level. Give out list of normalization factors."""
    m = get_experimental_matrix(bams, bed)
    d = zip(m.types, m.names)
    d = map(lambda x: x[1], filter(lambda x: x[0] == 'reads', d)) #list of names which are reads
    
    regions = m.objectsDict['housekeep'] #GenomicRegionSet containing housekeeping genes
         
    covs = []
    for cond in d:
        bam_path = m.files[cond]
        c = CoverageSet(cond, regions) 
        c.coverage_from_bam(bam_file=bam_path)
        c.genomicRegions.sort()
        covs.append(c)
    
    #create matrix sample x gene for signal
    signals = [[sum(covs[k].coverage[i])+1 for i in range(len(covs[k].genomicRegions))] for k in range(len(covs))]
    assert len(covs) > 0
    gene_names = [covs[0].genomicRegions[i].name for i in range(len(covs[0].genomicRegions))]
    
    colnames = gene_names
    d = np.matrix(signals, dtype=float)
    samples = map(lambda x: os.path.splitext(os.path.basename(x))[0], bams)
    #print("samples: %s" %",".join(map(lambda x: os.path.splitext(os.path.basename(x))[0], bams)))
    if verbose:
        print("-Housekeeping gene matrix (columns-genes, rows-samples)", file=sys.stderr)
        print(d, file=sys.stderr)
        print("", file=sys.stderr)
    
    if verbose or report:
        #output R code to check wether gene give same signal
        get_factor_matrix(d, colnames, folder, samples, verbose, report)
        #output_R_file(name, res, colnames)
    
    #print("factors")
    return get_factors(d), samples
    
def output(input, bed):
    factors, samples = norm_gene_level(input, bed, 'testname', True)
    
    for i in range(len(samples)):
        print(samples[i], factors[i], sep='\t')
        
if __name__ == '__main__':
    #bams = ['/home/manuel/test1.bam', '/home/manuel/test2.bam']
    #bams = ['/home/manuel/workspace/cluster_p/hematology/local/new_run/bam/32D_mm_BCRABL_H3K9ac_rep1.bam', '/home/manuel/workspace/cluster_p/hematology/local/new_run/bam/32D_mm_BCRABL_H3K9ac_rep2.bam', '/home/manuel/workspace/cluster_p/hematology/local/new_run/bam/32D_mm_BCRABL_IM_H3K9ac_rep1.bam', '/home/manuel/workspace/cluster_p/hematology/local/new_run/bam/32D_mm_BCRABL_IM_H3K9ac_rep2.bam', '/home/manuel/workspace/cluster_p/hematology/local/new_run/bam/32D_mm_JAK2VF_H3K9ac_rep1.bam', '/home/manuel/workspace/cluster_p/hematology/local/new_run/bam/32D_mm_JAK2VF_H3K9ac_rep2.bam', '/home/manuel/workspace/cluster_p/hematology/local/new_run/bam/32D_mm_JAK2VF_Rux_H3K9ac_rep1.bam', '/home/manuel/workspace/cluster_p/hematology/local/new_run/bam/32D_mm_JAK2VF_Rux_H3K9ac_rep2.bam', '/home/manuel/workspace/cluster_p/hematology/local/new_run/bam/32D_mm_LV_H3K9ac_forBCRABL_rep1.bam', '/home/manuel/workspace/cluster_p/hematology/local/new_run/bam/32D_mm_LV_H3K9ac_forBCRABL_rep2.bam', '/home/manuel/workspace/cluster_p/hematology/local/new_run/bam/32D_mm_LV_H3K9ac_forJAK2VF_rep1.bam', '/home/manuel/workspace/cluster_p/hematology/local/new_run/bam/32D_mm_LV_H3K9ac_forJAK2VF_rep2.bam']
    #bed = '/home/manuel/workspace/cluster_p/hematology/exp/exp16_check_housekeeping_genes/pot_housekeeping_genes_mm9.bed'
    
    #bams = ['/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/MPP_WT_H3K27ac_1.bam', '/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/MPP_WT_H3K27ac_2.bam', '/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/CDP_WT_H3K27ac_1.bam', '/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/CDP_WT_H3K27ac_2.bam']
    #bed = '/home/manuel/pot_housekeeping_genes_hg19.bed'
    
    #bams = ['/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC1_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC2_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC3_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC4_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC5_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/PBBA1_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/PBBA2_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/PBBA4_H3K27ac.bam']
    #bams = ['/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC1_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC2_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC4_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC5_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/PBBA1_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/PBBA2_H3K27ac.bam', '/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/PBBA4_H3K27ac.bam']
    #bams = ['/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/MPP_WT_H3K27ac_1.bam','/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/MPP_WT_H3K27ac_2.bam','/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/CDP_WT_H3K27ac_1.bam','/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/CDP_WT_H3K27ac_2.bam']
    
    #mm9
    bed = '/home/manuel/workspace/cluster_p/THOR/exp/exp68_THOR_genenorm_newgenes/hk_genes_new_promotor_mm9.bed'
    
    b_nestler_H3K36me3_sal = ['/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/nestler/H3K36me3_sal_rep1.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/nestler/H3K36me3_sal_rep2.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/nestler/H3K36me3_sal_rep3.bam']
    b_nestler_H3K36me3_coc = ['/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/nestler/H3K36me3_coc_rep1.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/nestler/H3K36me3_coc_rep2.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/nestler/H3K36me3_coc_rep3.bam']
    
    b_nestler_H3K36me1_sal = ['/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/nestler/H3K4me1_sal_rep1.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/nestler/H3K4me1_sal_rep2.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/nestler/H3K4me1_sal_rep3.bam']
    b_nestler_H3K36me1_coc = ['/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/nestler/H3K4me1_coc_rep1.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/nestler/H3K4me1_coc_rep2.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/nestler/H3K4me1_coc_rep3.bam']
    
    b_zenke_MPP = ['/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/MPP_WT_H3K27ac_1.bam','/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/MPP_WT_H3K27ac_2.bam']
    b_zenke_CDP = ['/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/CDP_WT_H3K27ac_1.bam','/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/CDP_WT_H3K27ac_2.bam']
    b_zenke_cDC = ['/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/cDC_WT_H3K27ac_1.bam','/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/cDC_WT_H3K27ac_2.bam']
    b_zenke_pDC = ['/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/pDC_WT_H3K27ac_1.bam','/home/manuel/workspace/cluster_p/dendriticcells/local/zenke_histones/bam/pDC_WT_H3K27ac_2.bam']
    
#     output(b_zenke_MPP + b_zenke_CDP, bed)
#     output(b_zenke_CDP + b_zenke_cDC, bed)
#     output(b_zenke_CDP + b_zenke_pDC, bed)
#     output(b_zenke_cDC + b_zenke_pDC, bed)
    
    
    #output(b_nestler_H3K36me3_sal + b_nestler_H3K36me3_coc, bed)
    #output(b_nestler_H3K36me1_sal + b_nestler_H3K36me1_coc, bed)
    
    
    #hg19
    #bed = '/home/manuel/pot_housekeeping_genes_hg19.bed'
    bed = '/home/manuel/workspace/cluster_p/THOR/exp/exp68_THOR_genenorm_newgenes/hk_genes_new_promotor_hg19.bed'
#     bed = '/home/manuel/hk.bed'
    
    b_payton_CC = ['/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC1_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC2_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC3_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC4_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/CC5_H3K27ac.bam']
    b_payton_PBBA = ['/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/PBBA1_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/PBBA2_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/PBBA4_H3K27ac.bam']
    b_payton_FL = ['/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/FL1_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/FL2_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/FL5_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/FL8_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/FL10_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/FL11_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/FL14_H3K27ac.bam','/home/manuel/workspace/cluster_p/allhoff/project_THOR/data/payton/FL16_H3K27ac.bam']
    
    b_blueprint_H3K27ac_monocyte = ['/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C000S5H2_H3K27ac_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C0010KH1_H3K27ac_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C001UYH2_H3K27ac_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C004SQH1_H3K27ac_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00BWXH1_H3K27ac_monocyte.bam']
    b_blueprint_H3K27ac_macrophage = ['/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C005VGH1_H3K27ac_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S001S7H1_H3K27ac_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S0022IH1_H3K27ac_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00390H1_H3K27ac_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00BYTH1_H3K27ac_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00C0JH1_H3K27ac_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00NM5H1_H3K27ac_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00NN3H1_H3K27ac_macrophage.bam']
    b_blueprint_H3K4me1_monocyte = ['/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C000S5H2_H3K4me1_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C0010KH1_H3K4me1_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C001UYH2_H3K4me1_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C004SQH1_H3K4me1_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00BWXH1_H3K4me1_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00NJBH1_H3K4me1_monocyte.bam']
    b_blueprint_H3K4me1_macrophage = ['/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C005VGH1_H3K4me1_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S001S7H1_H3K4me1_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S0022IH1_H3K4me1_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00390H1_H3K4me1_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00BXVH1_H3K4me1_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00BYTH1_H3K4me1_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00C0JH1_H3K4me1_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00NK9H1_H3K4me1_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00NM5H1_H3K4me1_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00NN3H1_H3K4me1_macrophage.bam']
    b_blueprint_H3K4me3_monocyte = ['/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C000S5H2_H3K4me3_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C0010KH1_H3K4me3_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C001UYH2_H3K4me3_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C004SQH1_H3K4me3_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00BWXH1_H3K4me3_monocyte.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00NJBH1_H3K4me3_monocyte.bam']
    b_blueprint_H3K4me3_macrophage = ['/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/C005VGH1_H3K4me3_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S001S7H1_H3K4me3_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S0022IH1_H3K4me3_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00390H1_H3K4me3_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00BXVH1_H3K4me3_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00BYTH1_H3K4me3_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00C0JH1_H3K4me3_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00NK9H1_H3K4me3_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00NM5H1_H3K4me3_macrophage.bam','/home/manuel/workspace/cluster_p//blueprint/raw/new_run/bams/S00NN3H1_H3K4me3_macrophage.bam']
    
    
    output(b_payton_FL + b_payton_CC, bed)
    #output(b_payton_CC + b_payton_PBBA, bed)
      
      
#     output(b_blueprint_H3K27ac_monocyte + b_blueprint_H3K27ac_macrophage, bed)
#     output(b_blueprint_H3K4me1_monocyte + b_blueprint_H3K4me1_macrophage, bed)
#     output(b_blueprint_H3K4me3_monocyte + b_blueprint_H3K4me3_macrophage, bed)
    
    
    
    



import sys
#seq = [(1,1),(4,1),(8,1),(51,1),(4,1),(5,1),(4,1),(6,1),(4,1),(1,1),(100,1),(2,1),(5,1),(3,1),(4,1),(1,1),(10,1),(2,1),(5,1),(3,1)]
S,p,q = [],[],[]
prefixW = []
prefixD = []

def setU_i(upperBound):
    n = len(seq)
    j, U = n-1, [-1]*n
    for i in range(n-1, -1, -1):
        while width(i, j) > upperBound:
            j -= 1
        if j >= i: #fuer universal
            U[i] = j
        
    return U

def setL_i(lowerBound):
    n = len(seq)
    j, L = 0, [-1]*n
    for i in range(0, n):
        while j < n and width(i, j) < lowerBound:
            j += 1
        if j == n: break
        L[i] = j
    return L

def width(x, y):
    #return sum( map( lambda x: x [1], seq[x:y+1]))
    if x >= len(seq): x = len(seq)-1
    if y >= len(seq): y = len(seq)-1
    if x == y:
        return float( seq[x][1] )
    else:
        if x == 0:
            tmp = 0
        else:
            tmp = prefixW[x-1]
        return float( prefixW[y] - tmp )

def density(x, y):
    if x >= len(seq): x = len(seq)-1
    if y >= len(seq): y = len(seq)-1
    if x>y: return 0
    #return float(sum( map( lambda x: x[0], seq[x:y+1]) )) / sum( map( lambda x: x[1], seq[x:y+1]) )
    if x == y:
        return float(seq[y][0] / width(x,y))
    else:
        if x == 0:
            tmp = 0
        else:
            tmp = prefixD[x-1]
        return float( (prefixD[y] - tmp) / width(x, y) )
    
def LMatchInitialize(x, y):
    global p, S
    l = u = b = p[b] = y
    for i in range(y, x, -1):
        p[i] = i
        while p[i] < y and density(i,p[i]) <= density(p[i]+1,p[p[i]+1]):
            p[i] = p[p[i]+1]
        if S[p[i]] == -1: S[p[i]] = []
        S[p[i]].insert(0, i)
    
    return l, u, b

def _getMinRemoveS(S_u, l, b):
    for el in S_u:
        if el >= l:
            erg = el
            break
    
    for k in S_u:
        if k > erg:
            S_u.remove(k)
    
    return erg
    
def LMatchFind(i, L, l, u, b, x):
    global S, p
    while l > 1 + max(x, L[i]):
        l -= 1
        if p[l] >= u:
            b = l
    if 1 + max(x, L[i]) >= len(seq): #extra
        l = len(seq)
    if l < 1 + max(x, L[i]): #extra
        l = 1 + max(x, L[i])
    while u >= l and density(i, b-1) > density(i, p[b]):
        u = b-1
        if u >= l:
            b = _getMinRemoveS(S[u], l, b)
    
    return u, l, b

def maxIndex(lowerBound):
    for j in reversed(range(len(seq))):
        if width(j, len(seq)-1) >= lowerBound:
            return j

def maxDensity(g):
    max, pos = -1, -1
    for i in range(len(g)):
        if density(i,g[i]) > max:
            max = density(i,g[i]) 
            pos = (i,g[i]+1)
    return max, pos
    
def MaximumDensitySegmentL(lowerBound, x, y):
    global S, p
    L = setL_i(lowerBound)
    l, u, b = LMatchInitialize(x,y)
    s = maxIndex(lowerBound)
    g = [] 
    for i in range(s,-1,-1):
        if L[i] == len(seq):
            g.append( (i,len(seq)) )
        else:
            goodPartner = LMatchFind(i, L, l, u, b, x)
            g.append( (i, goodPartner ) )
    return maxDensity(g)
    
            
def UMatchInitialize(x, y):
    global q
    u = y
    for i in range(x+1, y+1):
        q[i] = i
        while q[i] > x and q[q[i]-1] != -1 and density(q[q[i]-1], q[i]-1) <= density(q[i], i):
            q[i] = q[q[i]-1]
    return u

def UMatchFind(i, u, U, x):
    while u > U[i]:
        u -= 1
    while u > x and density(i, q[u]-1) > density(i, u):
        u = q[u]-1
    return u
    
def _getPosibleBlock(L, U): #fuer universal
    for i in range(len(L)):
        if L[i] <= U[i]:
            return L[i], U[i], i
        else:
            print("H")
    
def ConstructBlocks(lowerBound, upperBound):
    blocks = []
    L, U = setL_i(lowerBound), setU_i(upperBound)
    s, i = maxIndex(lowerBound), 0
    #fuer universal: x, y = L[0], U[0]
    x, y, i = _getPosibleBlock(L,U) #fuer universal
    blocks.append((x,y))
    while y < U[s]:
        x = y+1
        while (i < s and L[i] < x) or L[i] > U[i]:
            i += 1
        y = U[i]
        blocks.append((x,y)) 
    return blocks

def _getNextBlock(blocks, blocksPara):
    (x,y) = blocks.pop()
    l, u, b, v = blocksPara.pop()
    return x, y, l, u, b, v

def _LUNotPossible(L, U): #fuer universal
    notPossible = 1
    for i in range(len(L)):
        if L[i] == -1: continue
        if L[i] <= U[i]:
            notPossible = 0
    return notPossible
          
def MaximumDensitySegmentLU(lowerBound,upperBound):
    L, U = setL_i(lowerBound), setU_i(upperBound)
    if _LUNotPossible(L,U): #fuer universal
        return -1, (-1,-1)
    blocks = ConstructBlocks(lowerBound, upperBound)
    #print(len(blocks))
    blocksPara = []
    s = maxIndex(lowerBound)
    g = [-1]*(s+1)
    for (i,j) in blocks:
        l, u, b = LMatchInitialize(i,j)
        v = UMatchInitialize(i,j)
        blocksPara.append((l,u,b,v))
    #print('t')
    x, y, l, u, b, v = _getNextBlock(blocks, blocksPara)
    
    for i in range(s,-1,-1):
        #print(len(blocks),file=sys.stderr)
        if L[i] > U[i]: continue #fuer universal
        if x > L[i] or y < L[i]:
            xOld, vOld = x, v 
            x, y, l, u, b, v = _getNextBlock(blocks, blocksPara)
            
        g[i], l, b = LMatchFind(i, L, l, u, b, x)
        u = g[i]
        if U[i] > y:
            alt = UMatchFind(i, vOld, U, xOld)
            vOld = alt
            if density(i,alt) >= density(i,g[i]):
                g[i] = alt

    return maxDensity(g)

def testNaiv(seqTmp, l, u):
    global seq, prefixW, prefixD
    seq, maxF, pos, prefixW, prefixD = seqTmp, -1, (-1,-1), [], []
    getPrefixWD()
    for i in range(0,len(seq)):
        j = i
        #mindest groesse fuer j
        while width(i,j) < l and j < len(seq):
            j += 1
        if j >= len(seq): break
        #j vergroessern
        while width(i,j) <= u:
            if j >= len(seq): break
            if density(i,j) > maxF:
                maxF, pos = density(i,j), (i,j)
            
            j += 1
    
    return (maxF, pos)

def _naivLU(l): #fuer universal
    maxF, pos = -1, (-1,-1)
    for i in range(len(seq)):
        if width(i,i) == 1 and density(i,i) > maxF:
            maxF, pos = density(i, i), (i,i)
    return maxF, pos

def getPrefixWD():
    global prefixW, prefixD
    prefixW.append(seq[0][1])
    prefixD.append(seq[0][0])
    for i in range(1,len(seq)):
        prefixW.append(seq[i][1]+prefixW[i-1])
        prefixD.append(seq[i][0]+prefixD[i-1])


def AlgGoldwasser(seqTmp, lowerBound, upperBound):
    """Find maximum density segment. Give sequence [(value, width)], and
    upper/lower bound for window to search in.
    Return tuple (max. density, c) with c = (s1,s2) tuple with start- and
    end coordinates of window (s1 including, s2 excluding)."""
    global seq, S, p, q, prefixW, prefixD
    seq, prefixW, prefixD = seqTmp, [], []
    S, p, q = [-1]*len(seq), [-1]*len(seq), [-1]*len(seq)
    getPrefixWD()
    #print(prefixW, prefixD)
    
    if lowerBound > upperBound:
        raise Exception('lower bound > upper bound')
    elif seq == []:
        raise Exception('empty sequence')
    elif lowerBound > len(seq):
        raise Exception('wrong bounds')
    elif lowerBound == upperBound == 1:
        return _naivLU(lowerBound)
    return MaximumDensitySegmentLU(lowerBound,upperBound)
    

    
if __name__ == '__main__':
    #global seq, S, p, q, prefixW, prefixD
    #prefixW,prefixD = [], []
    seq = [(2,1),(5,1),(8,1),(4,1),(3,1),(6,1),(6,1), (0,1), (0,1), (0,1), (0,1), (0,1), (9,1),(3,1),(7,1),(1,1),(2,1),(4,1),(3,1)]

    
    #S, p, q = [-1]*len(seq),[-1]*len(seq), [-1]*len(seq)
    #getPrefixWD()
    #seq = [(2,1),(5,1),(8,1),(4,1),(3,1),(6,1),(6,1),(9,1),(3,1),(7,1),(1,1),(2,1),(4,1),(3,1)]
    lowerBound, upperBound = 5, 6
    
    #a, b , c = LMatchInitialize(0,13)
    #print('t')
    #U = setU_i(upperBound)
    #L = setL_i(lowerBound)
    
    #S, p, q = [-1]*len(seq),[-1]*len(seq), [-1]*len(seq)
    print(seq)
    b = AlgGoldwasser(seq, lowerBound, upperBound)
    print(b)
    
    #print(testNaiv(seq, lowerBound, upperBound))
    


"""
GenomicVariantSet
===================
GenomicVariantSet represents list of GenomicVariant.

"""

from __future__ import print_function
from rgt.GenomicVariant import GenomicVariant
from rgt.GenomicRegionSet import GenomicRegionSet
import vcf

class GenomicVariantSet(GenomicRegionSet):
    """*Keyword arguments:*

        - vcf_path -- VCF file
        - name -- name
    """
    
    def __init__(self, vcf_path = None, name='GenomicVariantSet'):
        """Initialize"""
        GenomicRegionSet.__init__(self, name = name)
        if vcf_path:
            self.read_vcf(vcf_path)
    
    def sort(self):
        """Sort elements by criteria defined by GenomicVariant.
        
        .. note:: By default, the genomic position is used as sorting criteria.
        
        """
        self.sequences.sort(cmp = GenomicVariant.__cmp__)
        self.sorted = True
         
    def read_vcf(self, vcf_path):
        """
        Read SNPs and InDels from a VCF file.
        
        *Keyword arguments:*

        - vcf_path -- VCF file
        
        .. note:: vcf_path can also be defined in the initialization.
        
        *Example:*
            We load a VCF file::
            
            >>>from rgt.GenomicVariantSet import GenomicVariantSet
            >>>snps_sample1 = GenomicVariantSet('snps.vcf', name='sample1')
        """
    
        self.reader = vcf.Reader(open(vcf_path), 'r')
         
        self.metadata = self.reader.metadata
        self.infos = self.reader.infos
        self.filters = self.reader.filters
        self.formats = self.reader.formats
        self.samples = self.reader.samples
         
        for v in self.reader:
            variant = GenomicVariant(v.CHROM, v.POS, v.REF, v.ALT, v.QUAL, filter = v.FILTER, id = v.ID, \
                                     info = v.INFO, format = v.FORMAT, genotype = v.genotype, samples = v.samples)
            self.add(variant)
    
    def write_vcf(self, vcf_path):
        """
        Write VCF file.
        
        *Keyword arguments:*

        - vcf_path -- VCF file
        
        """
        
        if not self.reader:
            raise Exception("No data available")
         
        writer = vcf.Writer(open(vcf_path, 'w'), self.reader)
        for v in self.sequences:
            record = vcf.model._Record(v.chrom, v.pos, v.id, v.ref, v.alt, v.qual, [], v.info, v.format, [], v.samples)
            writer.write_record(record)
 
    def filter_dbSNP(self):
        """Filter for dbSNP.
        
        .. note:: the vcf file must already contain the dbSNP annotation.
        
        """
        
        self.sequences = filter(lambda x: 'DB' not in x.info.keys(), self.sequences)
    
    def filter(self, at, op, t):
        """
        Filter for attributes. 
        
        *Keyword arguments:*

        - at -- VCF file
        - op -- operation to perform
        - t -- threshold
        
        *Example:*
            We load a VCF file::
            
            >>>from rgt.GenomicVariantSet import GenomicVariantSet
            >>>snps_sample1 = GenomicVariantSet('snps.vcf', name='sample1')
            
            And we filter by the mapping quality::
             
            >>>snps_sample1.filter(at='MQ', op'>', t=30)
            
            The mapping quality is tagged as MQ in the VCF file. We only want to keep SNPs that have a mapping quality higher than 30.
            
            .. note:: operation <op> and threhold <t> depend on the filtering tag <at>
        
        """
        self.sequences = filter(lambda x: eval(str(x.info[at]) + op + str(t)), self.sequences)
    
    def _reconstruct_info(self, GenomicRegionSet):
        """Reconstruct all information for GenomicVariantSet that get lost when using a GenomicRegionSet method"""
        tmp_sequences = []
        for genomic_region in GenomicRegionSet:
            c, p = genomic_region.chrom, genomic_region.initial
            for var in self.sequences:
                if var.chrom == c and var.pos == p: #1:1 mapping
                    tmp_sequences.append(var)
                    break
        
        return tmp_sequences
    
    def subtract(self, x):
        """
        Subtract GenomicVariantSet.
        
        *Keyword arguments:*

        - x -- instance of GenomicVariantSet which is subtracted
        
        """
        tmp = GenomicRegionSet.subtract(self, x, whole_region=False)
        self.sequences = self._reconstruct_info(tmp)
    
    def intersect(self, x):
        """
        Intersect GenomicVariantSet.
        
        *Keyword arguments:*

        - x -- instance of GenomicVariantSet
        
        """
        
        tmp = self._intersect(x)
        self.sequences = self._reconstruct_info(tmp)
    
    def _intersect(self, y, rm_duplicates=False):
        """Return the overlapping regions with three different modes.
                   
        (mode = OverlapType.ORIGINAL)
        Return the regions of original GenomicRegionSet which have any intersections with y.
        
            Keyword arguments:
            y -- the GenomicRegionSet which to compare with
            
            Return:
            z -- the regions of original GenomicRegionSet which have any intersections with y
            
            Graphical explanation:
            self       ----------              ------
            y              ----------                    ----
            Result     ----------
            
        """
        a = self
        b = y
        
        z = GenomicRegionSet(a.name + ' + ' + b.name)
        # XXX - someone putted an special symbol and spaces in the name! this is used as file name, never use strange characters.
        if len(a) == 0 or len(b) == 0: return z
        
        else:
            # If there is overlap within self or y, they should be merged first. 
            if a.sorted == False: a.sort()
            if b.sorted == False: b.sort()
            
            iter_a = iter(a)
            s = iter_a.next()
            last_j = len(b)-1
            j = 0
            cont_loop = True
                 
            ########################### OverlapType.ORIGINAL ###################################
            while cont_loop:
                #print(str(s),"\t",str(b[j]))
                # When the regions overlap
                if s.overlap(b[j]):
                    z.add(s)
                    
                    
                    try: s = iter_a.next()
                    except: cont_loop = False
                    
                elif s < b[j]:
                    try: s = iter_a.next()
                    except: cont_loop = False
                elif s > b[j]:
                    if j == last_j: cont_loop = False
                    else: j = j + 1
                else:
                    try: s = iter_a.next()
                    except: cont_loop = False
                        
            if rm_duplicates: 
                z.remove_duplicates()
            
            return z
    
    
if __name__ == '__main__':
    s = GenomicVariantSet('/home/manuel/data/humangenetics/01_S1_L001_R1_001.filtered.vcf')
    b = GenomicVariantSet('/home/manuel/data/humangenetics/K28_S8_L001_R1_001.filtered.vcf')
    print(len(s))
    s.subtract(b)
    
    print(len(s))
    #print(b.sequences[:10])
    #print(c.sequences[:10])
    print(len(s))
    s.filter('MQ', '>=', 40.0)
    print(len(s))
    #s.filter_DB()
    #print(len(s))
    #s.write_vcf('/home/manuel/1.vcf')
    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
THOR detects differential peaks in multiple ChIP-seq profiles between
two distinct biological conditions.

Copyright (C) 2014-2016 Manuel Allhof (allhoff@aices.rwth-aachen.de)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

@author: Manuel Allhoff
"""

from __future__ import print_function
import sys
import numpy as np
from math import fabs
import warnings
warnings.filterwarnings('error')

def _count(posts):
    c_1, c_2 = 1, 1
    
    for s0, s1, s2 in posts:        
        if s0 > 0.5:
            c_1 += 0
            c_2 += 0
        elif s1 >= s2:
            c_1 += 1
        elif s2 > s1:
            c_2 += 1
    return c_1, c_2

def _valid_posteriors(posteriors, obs, dim):
    
    warnings.filterwarnings('error')
    
    for i in range(len(obs)):
        state_1 = False
        c1, c2 = np.mean(obs[i][:dim[0]]), np.mean(obs[i][dim[0]:]) #counts of samples

        if posteriors[i][0] > 0.5: 
            state_1 = True
            
        if c1 > c2: #state 1
            if fabs(posteriors[i][2] - 1) < 1e-200:
                posteriors[i] = np.array([1, 0, 0])
            else:
                if not state_1 and posteriors[i][2] > posteriors[i][1]:
                    try:                        
                        post_s2 = 0
                        post_s0 = posteriors[i][0] / (posteriors[i][0] + posteriors[i][1])
                        post_s1 = posteriors[i][1] / (posteriors[i][0] + posteriors[i][1])
                        posteriors[i] = np.array([post_s0, post_s1, post_s2])
                    except RuntimeWarning:
                        print(posteriors[i], c1, c2, file=sys.stderr)
        
        if c2 > c1: #state 2
            if fabs(posteriors[i][1] - 1) < 1e-200:
                posteriors[i] = np.array([1, 0, 0])
            else:
                if not state_1 and posteriors[i][1] > posteriors[i][2]:
                    try:
                        post_s1 = 0
                        post_s0 = posteriors[i][0] / (posteriors[i][0] + posteriors[i][2])
                        post_s2 = posteriors[i][2] / (posteriors[i][0] + posteriors[i][2])
                        posteriors[i] = np.array([post_s0, post_s1, post_s2])
                    except RuntimeWarning:
                        print(posteriors[i], c1, c2, file=sys.stderr)
    
    return posteriors




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
THOR detects differential peaks in multiple ChIP-seq profiles associated
with two distinct biological conditions.

Copyright (C) 2014-2016 Manuel Allhoff (allhoff@aices.rwth-aachen.de)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

@author: Manuel Allhoff
"""

from __future__ import print_function
import sys
import string
import numpy as np
from scipy.stats import binom
from hmmlearn.hmm import _BaseHMM
from help_hmm import _valid_posteriors

def get_init_parameters(s1, s2, **info):
    n_ = np.array([info['count'], info['count']])
    #get observation that occurs most often:
    m_ =[float(np.argmax(np.bincount(map(lambda x: x[0], s1)))), float(np.argmax(np.bincount(map(lambda x: x[1], s2)))) ]
    p_ = [[-1,-1,-1],[-1,-1,-1]] #first: 1. or 2. emission, second: state
    
    p_[0][0] = 1. / n_[0]
    p_[1][0] = 1. / n_[1]
    p_[0][1] = m_[0] / n_[0]
    p_[1][1] = p_[1][0]
    p_[0][2] = p_[0][0]
    p_[1][2] = m_[1] / n_[1]
    
    return np.asarray(n_), np.asarray(p_)

class BinomialHMM(_BaseHMM):
    def __init__(self, n, p, dim_cond_1, dim_cond_2, init_state_seq=None, n_components=2, covariance_type='diag', startprob=None,
                 transmat=None, startprob_prior=None, transmat_prior=None,
                 algorithm="viterbi", means_prior=None, means_weight=0,
                 covars_prior=1e-2, covars_weight=1,
                 random_state=None, n_iter=10, thresh=1e-2,
                 params=string.ascii_letters,
                 init_params=string.ascii_letters):
    
        _BaseHMM.__init__(self, n_components, startprob, transmat,
                          startprob_prior=startprob_prior,
                          transmat_prior=transmat_prior, algorithm=algorithm,
                          random_state=random_state, n_iter=n_iter,
                          thresh=thresh, params=params,
                          init_params=init_params)
        
        self.dim = [dim_cond_1, dim_cond_2] #dimension of one emission
        self.n = n
        self.p = p
        self.n_features = 2 #emission dimension
        self.init_state_seq = init_state_seq
        self.count_s1, self.count_s2 = 0, 0
        self.lookup_logpmf = {}

    def _compute_log_likelihood(self, X):
        res = []
        
        for x in X: #over all observations
            row = []
            for i in range(self.n_components): #over number of HMM's state
                r_sum = 0
                for j in range(self.n_features): #over dim
                    it = range(self.dim[0]) if j == 0 else range(self.dim[0], self.dim[0] + self.dim[1]) #grab proper observation
                    for k in it:
                        index = (int(x[k]), self.p[j][i], self.n[j])
                        if not self.lookup_logpmf.has_key( index ):
                            self.lookup_logpmf[index] = binom.logpmf(x[k], self.n[j], self.p[j][i])
                        r_sum += self.lookup_logpmf[index]
                row.append(r_sum)
        
            res.append(row)
        
        return np.asarray(res)
    

    def _generate_sample_from_state(self, state, random_state=None):
        output = []
        for i, d in enumerate(self.dim):
            for _ in range(d):
                output.append( binom.rvs(self.n[i], self.p[i][state]) )
        
        return np.asarray(output)
            
    def _initialize_sufficient_statistics(self):
        stats = super(BinomialHMM, self)._initialize_sufficient_statistics()
        stats['post'] = np.zeros([self.n_components])
        stats['post_emission'] = np.zeros([self.n_features, self.n_components])
        return stats
    
    def _help_accumulate_sufficient_statistics(self, obs, stats, posteriors):
        for t, symbol in enumerate(obs):
            pot_it = [range(self.dim[0]), range(self.dim[0], self.dim[0] + self.dim[1])] #consider both classes
            for j, it in enumerate(pot_it):
                for i in it:
                    stats['post'] += posteriors[t]
                    stats['post_emission'][j] += posteriors[t] * symbol[i]
            
        stats['posterior'] = np.copy(posteriors)
        
    def _accumulate_sufficient_statistics(self, stats, obs, framelogprob,
                                      posteriors, fwdlattice, bwdlattice,
                                      params):
        super(BinomialHMM, self)._accumulate_sufficient_statistics(
            stats, obs, framelogprob, posteriors, fwdlattice, bwdlattice,
            params)
        
        posteriors = _valid_posteriors(posteriors, obs, self.dim)
        self._help_accumulate_sufficient_statistics(obs, stats, posteriors)

    def _add_pseudo_counts(arr):
        if type(arr) is np.ndarray:
            tmp = np.array([1e-323 if x < 1e-323 else x for x in arr], np.float64)
            # tmp2 = np.array([1.0 - 1.0e-5 if x == 1.0 else x for x in tmp], np.float64)
            return tmp
        else:
            tmp = 1e-323 if arr < 1e-323 else arr
            # tmp2 = 1.0 - 1.0e-10 if tmp == 1.0 else tmp
            return tmp

    def _help_do_mstep(self, stats):
        for i in range(self.n_features):
            self.p[i] = stats['post_emission'][i] / (self.n[i] * self._add_pseudo_counts(stats['post']))
            print('help_m_step', i, stats['post_emission'][i], stats['post'], self.p[i], file=sys.stderr)
        
    def _do_mstep(self, stats, params):
        super(BinomialHMM, self)._do_mstep(stats, params)
        self._help_do_mstep(stats)

        self.p[0,0] = self.p[1,0]
        self.p[0,1] = self.p[1,2]
        self.p[1,1] = self.p[0,2]
    
if __name__ == '__main__':
    p_ = np.array([[0.01, 0.8, 0.1], [0.01, 0.1, 0.8]])
    n_ = np.array([100, 100])
    
    m = BinomialHMM(n_components=3, p = p_, startprob=[1,0,0], n = n_, dim_cond_1=2, dim_cond_2=4)
    
    X, Z = m.sample(100) #returns (obs, hidden_states)
    
    p_ = np.array([[0.1, 0.7, 0.3], [0.1, 0.2, 0.9]])
    n_ = np.array([100, 100])
    
    m2 = BinomialHMM(n_components=3, n=n_, p=p_, dim_cond_1=2, dim_cond_2=4)
    #cProfile.run("m2.fit([X])")
    
    m2.fit([X])
    e = m2.predict(X)
    print(m2.p)
    for i, el in enumerate(X):
        print(el, Z[i], e[i], Z[i] == e[i], sep='\t')
    
#    logprob, posteriors = m2.eval(X)
#    print('logprob:', logprob)
#    print('posteriors:', posteriors)
    
#     print('estim. states ', m2.predict(X))
#     print(m2.predict_proba(X))
#     print(m2.n)
#     print(m2.p)
#     print(m2._get_transmat())
#     init_state = m2.predict(X)
#     m3 = BinomialHMM2d3s(n_components=3, n=n_)
#     m3.fit([X], init_params='advanced')
#     print(m3._get_transmat())
#     print(m3.p)
#     m2.eval(X)
    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
THOR detects differential peaks in multiple ChIP-seq profiles associated
with two distinct biological conditions.

Copyright (C) 2014-2016 Manuel Allhoff (allhoff@aices.rwth-aachen.de)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

@author: Manuel Allhoff
"""

from __future__ import print_function
import re
import numpy as np
from rgt.Util import Html
from collections import OrderedDict
from os import path
# import sys
from datetime import datetime


class Tracker:
    data = []

    def __init__(self, p, bamfiles, genome, chrom_sizes, dims, inputs, options, version):
        self.file = open(p, 'w')
        self.bamfiles = bamfiles
        if genome is None:
            self.genome = "None"
        else:
            self.genome = genome
        self.chrom_sizes = chrom_sizes
        self.dims = dims
        self.inputs = inputs
        self.samples = map(lambda x: path.splitext(path.basename(x))[0], bamfiles)
        self.options = options
        self.version = version
    
    def write(self, text, header):
        if header:
            self.file.write('#' + header + '\n')
        if isinstance(text, np.ndarray):
            text = re.sub(' +',' ', str(text).replace('\n ', '\n').replace('[','').replace(']','').strip().\
                          replace('\n ', '\n').replace('\n ', '\n').replace('\n ', '\n'))
        else:
            if isinstance(text, list):
                text = " ".join(text)
            if isinstance(text, int) or isinstance(text, float):
                text = str(text)
            
        self.file.write(text + '\n')
        
        tmp = text.replace('\n', ' ').split(' ')
        if len(tmp) == 1:
            text = tmp[0]
        else:
            new = []
            for i, el in enumerate(tmp):
                i += 1
                new.append(el)
                if i > 0 and i % 3 == 0 and i < len(tmp) and len(tmp) % 3 == 0:
                    new.append('<br>')
                
            text = " ".join(new)
        self.data.append((header, text))

    def _read_hk(self, p):
        d = []
        if path.isfile(p):
            with open(p) as f:
                for line in f:
                    line = line.split(" ")
                    d.append([line[0], str(float(line[1]))])
        return d
    
    def make_hmm(self, html):
        d_all = []
        for el in self.data:
            if not el[0].startswith('Neg') and not el[0].startswith('Parameters') and\
            not el[0].startswith('Ext') and not el[0].startswith('Scaling'):
                d_all.append((el[0], el[1]))
        
        html.add_list([el[0] + '<br>' + el[1] for el in d_all])
        
        info = "THOR uses an HMM to segment the ChIP-seq signals. The HMM's emission is a Negative Binomial distribution. Inital\
        values (before HMM training) and final values (after HMM training) of the parameters mu and alpha are given. Furthermore,\
        the transition matrix is given. The rows and columns of the matrix describe: background state, DP found in first \
        condition and DP found in second condition. See [1] for further details about the HMM training."
        self._write_text(html, info)
        
    def make_pre_post(self, html):
        d_all = []
        for el in self.data:
            if el[0].startswith('Neg') or el[0].startswith('Parameters'):
                d_all.append((el[0], el[1]))
        
        html.add_list([el[0] + '<br>' + el[1] for el in d_all])
        
        info = "THOR's p-value calculation is based on a Negative Binomial distribution with \
        parameter mu which gives the location and paramter alpha which gives the dispersion. <br>Moreover, THOR uses a\
        polynomial function of degree 2 with parameters <i>a</i> and <i>c</i> to empirically describe the mean-variance relationship in\
        the data. <br><b>Note:</b> Parameter <i>a</i> describes the function's quadratic characteristic and can be used as indicator \
        for overdispersion. In our experience, for experiments with high <i>a</i> (higher than 1e-2) it is better\
        to normalize with housekeeping genes (see [1] for more details)."
        self._write_text(html, info)
    
    def make_ext_scaling_table(self, html):
        """make table: sample, ext, scaling"""
        d_all = []
        for el in self.data:
            if el[0].startswith('Ext'):
                d_all.append(el[1])
            if el[0].startswith('Scaling'):
                d_all.append(el[1])
        
        exts = d_all[0].split(' ')
        factors = d_all[1].split(' ')
        
        l = []
                
        for i in range(len(exts)):
            l.append([self.samples[i], exts[i], factors[i]])
        
        html.add_zebra_table(header_list=['Sample', 'Extension Size', 'Scaling Factor'], col_size_list=[1,150,150], type_list='sss', data_table=l, auto_width = True)
        
        info = "Since the Chip-seq protocol makes it necessary to extend the experiment's reads, we follow [2] and define a strand cross-correlation function to derive the\
        extension size from it.<br> To bring all ChIP-seq profiles to the same scale, THOR normalizes each sample with a scaling factor. \
        The scaling factor is either based on TMM [3] or the house-keeping gene approach [1]."
        self._write_text(html, info)
  
    def make_ext_config(self, html):
        """make table about configuration: feature, path"""
        b = []
        for el in self.bamfiles:
            b += [el, '<br>']
        
        b = b[:len(b) - 1]
        b = " ".join(b)
        
        if self.options.housekeeping_genes:
            norm = 'Housekeeping Genes approach'
        elif self.options.scaling_factors_ip:
            norm = 'Predefined Values'
        else:
            norm = 'TMM'
        
        d = [['Name', path.splitext(path.basename(self.options.name))[0]], ['time', str(datetime.now()).split(".")[0]], ['BAM files', b], ['genome', self.genome], ['chromosome sizes', self.chrom_sizes],\
             ['Normalization Strategy', norm], ['version', self.version], ['merge DPs', str(self.options.merge)], ['p-value cutoff', str(self.options.pcutoff)],\
             ['deadzones', str(self.options.deadzones)]]
        
        if self.inputs:
            a = []
            for el in self.inputs:
                a += [el, '<br>']
            
            a = a[:len(a) - 1]
            a = " ".join(a)
        
            d.append(['BAM input-DNA files', a])
        
        html.add_zebra_table(header_list=['Feature', ''], col_size_list=[190, 1000], type_list='ss', data_table=d, cell_align='left')
    
    def _write_text(self, html, t):
        r = "<table style=\"width:90%\"  align=\"center\"  cellpadding=\"0\" cellspacing=0>"
        r += "<tr><td>"
        #t = t.replace('<br>', '<br>' + '&nbsp;'*9)
        #t = t.replace('<p>', '<p>' + '&nbsp;'*9)
        #t = '<p>' + '&nbsp;'*10 + t + '<br>'
        r += t
        r += "</td></tr>"
        r += "</table>"
        html.add_free_content([r])
    
    def make_html(self):
        html_header = "THOR"
        from rgt.THOR.dpc_help import FOLDER_REPORT

        #Links
        links_dict = OrderedDict()
        links_dict['Experimental Configuration'] = 'index.html#extinfo'
        links_dict['Sample Information'] = 'index.html#sampleinfo'
        links_dict['HMM Information'] = 'index.html#hmminfo'
        links_dict['Mean Variance Function Estimate'] = 'index.html#mvfunction'
        
        p = path.join(FOLDER_REPORT, 'pics/fragment_size_estimate.png')
        if path.isfile(p):
            links_dict['Fragment Size Estimate'] = 'index.html#fsestimate'

        p = path.join(FOLDER_REPORT, 'pics/data/sample.data')
        if path.isfile(p):
            links_dict['Housekeeping Gene Normalization'] = 'index.html#norm'
        
        links_dict['References'] = 'index.html#ref'
        links_dict['Contact'] = 'index.html#contact'

        # copy basic rgt logo, style etc to local directory inside report
        fig_path = path.join(FOLDER_REPORT, "fig")
        html = Html(name=html_header, links_dict=links_dict, fig_dir=fig_path, fig_rpath="fig")
        
        try:
            html.add_heading("Experimental Configuration", idtag='extinfo')
            self.make_ext_config(html)
        except:
            pass

        html.add_heading("Pre- and post-processing Features", idtag='prepostinfo')
        self.make_pre_post(html)
        
        try:
            html.add_heading("Sample Information", idtag='sampleinfo')
            self.make_ext_scaling_table(html)
        except:
            pass

        #Run Info
        try:
            html.add_heading("HMM Information", idtag='hmminfo')
            self.make_hmm(html)
        except:
            pass

        #Mean Variance Function
        try:
            p = path.join(FOLDER_REPORT, 'pics/mean_variance_func_cond_0_original.png')
            if path.isfile(p):
                html.add_heading("Mean Variance Function", idtag='mvfunction')
                html.add_figure(path.relpath(p, FOLDER_REPORT), align="left", width="45%",
                                more_images=['pics/mean_variance_func_cond_1_original.png'])
                info = "THOR uses a polynomial function to empirically describe the relationship between mean and variance in the data.\
                The data the plot is based on can be found at report/pics/data for further downstream analysis."
                self._write_text(html, info)
        except:
            pass

        #Fragment Size Estimate
        try:
            p = path.join(FOLDER_REPORT, 'pics/fragment_size_estimate.png')
            if path.isfile(p):
                html.add_heading("Fragment Size Estimate", idtag='fsestimate')
                html.add_figure(path.relpath(p, FOLDER_REPORT), align="left", width="45%")
                info = "THOR estimates the fragmentation sizes of each sample's reads. Here, the cross-correlation function [1] is shown. Their maxima give the\
                fragmentation extension sizes.<br> The data the plot is based on can be found at report/pics/data for further downstream analysis."
                self._write_text(html, info)
        except:
            pass

        #HK normalization
        try:
            p = path.join(FOLDER_REPORT, 'pics/data/gene.data')
            if path.isfile(p):
                d = self._read_hk(p)
                html.add_heading("Housekeeping Gene Normalization", idtag='norm')
                html.add_zebra_table(header_list=['gene', 'quality q'], col_size_list=[1,150], type_list='s'*len(d), data_table=d)
                info = "For active histone marks, housekeeping genes given by [4] can be used for normalization [1]. Here, the genes for the experiments are\
                evaluated. For each gene i, we estimate the normalization factors with gene i and without gene i and compute the sums of squared deviations q.\
                High values (higher than 2) indicate striking genes which should be considered to be left our for normalization.,<br> One can also \
                use other genes or regions for normalization.<br> The data the plot is based on can be found at report/pics/data for further downstream analysis."
                self._write_text(html, info)
                
            p = path.join(FOLDER_REPORT, 'pics/data/sample.data')
            if path.isfile(p):
                d = self._read_hk(p)
                html.add_zebra_table(header_list=['sample', 'quality p'], col_size_list=[1,150], type_list='s'*len(d), data_table=d)
                info = "We evaluate the effect of samples to the normalization factors. For sample j, we estimate the normalization factors with sample j\
                and without sample j and compute the sums of squared deviations p. High values (higher than 2) indicate striking samples which should be\
                considered to be left out for the analysis.<br> The data the plot is based on can be found at report/pics/data for further downstream analysis."
                self._write_text(html, info)
        except:
            pass

        html.add_heading("References", idtag='ref')
        info = "[1] M. Allhoff, J. F. Pires, K. Ser&eacute;, M. Zenke, and I. G. Costa. Differential Peak Calling of ChIP-Seq \
        Signals with Replicates with THOR. <i>submitted.</i> <br>\
        [2] A. Mammana, M. Vingron, and H.-R. Chung. Inferring nucleosome positions with their histone mark annotation from chip data. \
        Bioinformatics, 29(20):2547-2554, 2013. <br>\
        [3] M. D. Robinson and A. Oshlack. A scaling normalization method for differential expression analysis of RNA-seq data. \
        Genome Biology, 11(3):R25, 2010. <br>\
        [4] E. Eisenberg and E. Y. Levanon. Human housekeeping genes, revisited. Trends in genetics: TIG, 29(10):569-574, 2013."
        self._write_text(html, info)
        
        html.add_heading("Contact", idtag='contact')
        info = "If you have any questions, please don't hesitate to contact us: allhoff@aices.rwth-aachen.de"
        self._write_text(html, info)
        
        html.write(path.join(FOLDER_REPORT, "index.html"))



# Python Libraries
from __future__ import print_function
from __future__ import division
import re
import time
import numpy
import pickle
import fnmatch
import urllib2
import datetime
from shutil import copyfile
import multiprocessing.pool
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages

# Local Libraries
# Distal Libraries
from rgt.Util import Html
from rgt.CoverageSet import *
from rgt.ExperimentalMatrix import *
from rgt.AnnotationSet import AnnotationSet
from rgt.motifanalysis.Statistics import multiple_test_correction

# Local test
dir = os.getcwd()

###########################################################################################
#                    Universal functions
###########################################################################################


def print2(parameter, string):
    """ Show the message on the console and also save in a list for future backup. """
    print(string)
    parameter.append(string)


def unique(a):
    seen = set()
    return [seen.add(x) or x for x in a if x not in seen]


def purge(dir, pattern):
    for f in os.listdir(dir):
        if re.search(pattern, f):
            os.remove(os.path.join(dir, f))


def gen_tags(exps, tag):
    """Generate the unique tags from the EM according to the given tag. """
    if "factor" not in exps.fields:
        exps.add_factor_col()
    if tag == "reads":
        try:
            l = [exps.get_type(i, "factor") for i in exps.get_readsnames()]
        except:
            # print("You must define 'factor' column in experimental matrix for grouping.")
            sys.exit(1)
    elif tag == "regions":
        try:
            l = [exps.get_type(i, "factor") for i in exps.get_regionsnames()]
        except:
            # print("You must define 'factor' column in experimental matrix for grouping.")
            sys.exit(1)
    else:
        l = exps.fieldsDict[tag]
        try:
            l = exps.fieldsDict[tag]
        except:
            print('Cannot find the column "' + tag + '"')
            sys.exit(1)
    try:
        l.remove("ALL")
    except:
        pass
    return unique(l)


def tag_from_r(exps, tag_type, name):
    tags = []
    for t in tag_type:
        if t == "reads" or t == "regions": t = "factor"
        try:
            tags.append(exps.get_type(name, t))
        except:
            pass

    return [x for x in tags if x is not None]


def colormap(exps, colorby, definedinEM, annotation=None):
    """Generate the self.colors in the format which compatible with matplotlib"""
    if definedinEM:
        if colorby == "reads":
            colors = []
            for i in exps.get_readsnames():
                c = exps.get_type(i, "color")
                if c[0] == "(":
                    rgb = [eval(j) for j in c.strip('()').split(',')]
                    colors.append(rgb)
                else:
                    colors.append(c)
        elif colorby == "regions":
            colors = []
            for i in exps.get_regionsnames():
                c = exps.get_type(i, "color")
                if c[0] == "(":
                    rgb = [eval(j) for j in c.strip('()').split(',')]
                    colors.append([v / 255 for v in rgb])
                else:
                    colors.append(c)
        else:
            colors = []
            for i in exps.fieldsDict[colorby].values():
                c = exps.get_type(i[0], "color")
                if c[0] == "(":
                    rgb = [float(j) for j in c.strip('()').split(',')]
                    colors.append([v / 255 for v in rgb])
                else:
                    colors.append(c)

    else:
        if annotation:
            colors = plt.cm.Set1(numpy.linspace(0, 1, len(annotation))).tolist()
        else:
            # colors = [ 'lightgreen', 'pink', 'cyan', 'lightblue', 'tan', 'orange']
            # colors = plt.cm.jet(numpy.linspace(0.1, 0.9, len(gen_tags(exps, colorby)))).tolist()
            if colorby == "reads":
                ks = []
                for gr in exps.get_readsnames():
                    ks.append(exps.get_type(name=gr, field="factor"))
                n = len(set(ks))
            elif colorby == "regions":
                ks = []
                for gr in exps.get_regionsnames():
                    ks.append(exps.get_type(name=gr, field="factor"))
                n = len(set(ks))
            else:
                n = len(exps.fieldsDict[colorby].keys())
            # print(n)
            # colors = plt.cm.Spectral(numpy.linspace(0.1, 0.9, n)).tolist()
            colors = plt.cm.Set1(numpy.linspace(0, 1, n)).tolist()
    return colors


def colormaps(exps, colorby, definedinEM):
    """Generate a list of colormaps in the format which compatible with matplotlib"""
    if definedinEM:
        if colorby == "reads":
            colors = []
            for i in exps.get_readsnames():
                c = exps.get_type(i, "color")
                colors.append(exps.get_type(i, "color"))
        elif colorby == "regions":
            colors = []
            for i in exps.get_regionsnames():
                c = exps.get_type(i, "color")
                colors.append(exps.get_type(i, "color"))
        else:
            colors = [exps.get_type(i, "color") for i in exps.fieldsDict[colorby]]
    else:
        if colorby == "reads" or colorby == "regions":
            n = len(exps.fieldsDict["factor"].keys())
        else:
            n = len(exps.fieldsDict[colorby].keys())
        colors = plt.cm.Set1(numpy.linspace(0, 1, n)).tolist()


        # if len(exps.get_regionsnames()) < 20:
        #    colors = ['Blues', 'Oranges', 'Greens', 'Reds',  'Purples', 'Greys', 'YlGnBu', 'gist_yarg', 'GnBu',
        #              'OrRd', 'PuBu', 'PuRd', 'RdPu', 'YlGn', 'BuGn', 'YlOrBr', 'BuPu','YlOrRd','PuBuGn','binary']
        # else:
        # colors = plt.cm.Set2(numpy.linspace(0.1, 0.9, len(exps.get_regionsnames()))).tolist()
        #    colors = plt.cm.Spectral(numpy.linspace(0.1, 0.9, len(exps.get_regionsnames()))).tolist()
    return colors


def color_groupded_region(EM, grouped_region, colorby, definedinEM):
    """Generate the self.colors in the format which compatible with matplotlib"""
    if definedinEM:
        colors = OrderedDict()
        for ty in grouped_region.keys():
            for q in grouped_region[ty].keys():
                c = EM.get_type(q.name, "color")
                if c[0] == "(":
                    rgb = [eval(j) for j in c.strip('()').split(',')]
                    colors[q.name] = rgb
                else:
                    colors[q.name] = c
    else:
        colors = OrderedDict()
        qs = []
        if colorby == "regions":
            for ty in grouped_region.keys():
                for q in grouped_region[ty]:
                    qs.append(q.name)
            qs = list(set(qs))
            # Accent Spectral hsv Set1
            colormap = plt.cm.Spectral(numpy.linspace(0.1, 0.9, len(qs))).tolist()

            for i, q in enumerate(qs):
                colors[q] = colormap[i]
        else:
            types = EM.fieldsDict[colorby].keys()

            colormap = plt.cm.Spectral(numpy.linspace(0.1, 0.9, len(types))).tolist()

            for ty in grouped_region.keys():
                for q in grouped_region[ty]:
                    i = types.index(EM.get_type(q.name, colorby))
                    colors[q.name] = colormap[i]
    return colors


def output_array(array, directory, folder, filename):
    """ Write a txt file from the given array. """
    pd = os.path.join(dir, directory, folder)
    try:
        os.stat(os.path.dirname(pd))
    except:
        os.mkdir(os.path.dirname(pd))
    try:
        os.stat(pd)
    except:
        os.mkdir(pd)

    f = open(os.path.join(pd, filename), "w")
    for i, line in enumerate(array):
        f.write(("\t".join(str(j) for j in line)) + "\n")
    f.close()


def remove_duplicates(grouped_dict):
    for ty in grouped_dict:
        for r in grouped_dict[ty]:
            r.remove_duplicates()


def group_refque(rEM, qEM, groupby, rRegion=None, qRegion=None):
    """ Group regionsets of rEM and qEM according to groupby """
    groupedreference = OrderedDict()  # Store all bed names according to their types
    groupedquery = OrderedDict()  # Store all bed names according to their types
    if rRegion:
        rregs = rRegion
    else:
        rregs = rEM.get_regionsets()
    if qRegion:
        qregs = qRegion
    else:
        qregs = qEM.get_regionsets()

    if groupby:
        for r in rregs:
            ty = rEM.get_type(r.name, groupby)
            try:
                groupedreference[ty].append(r)
            except:
                groupedreference[ty] = [r]

        for q in qregs:
            ty = qEM.get_type(q.name, groupby)
            try:
                groupedquery[ty].append(q)
            except:
                groupedquery[ty] = [q]
    else:
        groupedreference[""] = rregs
        groupedquery[""] = qregs
    return groupedreference, groupedquery


def count_intersect(reference, query, mode_count="count", threshold=False):
    bed1 = copy.deepcopy(reference)
    bed2 = copy.deepcopy(query)
    if mode_count == "count":
        if threshold:
            if bed1.total_coverage() == 0:
                print(
                "\n ** Warning : " + bed1.name + " has no length (only points) for finding intersection with given threshold.")
                sys.exit(1)
            if bed2.total_coverage() == 0:
                print(
                "\n ** Warning : " + bed2.name + " has no length (only points) for finding intersection with given threshold.")
                sys.exit(1)
            if 50 >= threshold > 0:
                bed1.extend(-threshold, -threshold, percentage=True)
            elif threshold > 50 or threshold < 0:
                print("\n **** Threshold should be the percentage between 0 and 50. ****\n")
                sys.exit(1)
        ##if bed1.total_coverage() == 0: bed1.extend(0,1)
        # if bed2.total_coverage() == 0: bed2.extend(0,1)
        intersect_r = bed1.intersect(bed2, mode=OverlapType.OVERLAP)
        c_inter = len(intersect_r)
        intersect_1 = bed1.intersect(intersect_r, mode=OverlapType.ORIGINAL)
        c_12 = len(bed1) - len(intersect_1)
        intersect_2 = bed2.intersect(intersect_r, mode=OverlapType.ORIGINAL)
        c_21 = len(bed2) - len(intersect_2)
        # print(c_12, c_21, c_inter)
        return c_12, c_21, c_inter

    elif mode_count == "bp":
        intersect_r = bed1.intersect(bed2, mode=OverlapType.OVERLAP)
        len_inter = intersect_r.total_coverage()
        allbed1 = bed1.total_coverage()
        allbed2 = bed2.total_coverage()
        len_12 = allbed1 - len_inter
        len_21 = allbed2 - len_inter
        return len_12, len_21, len_inter


def value2str(value):
    if (isinstance(value, str)): return value
    if value == 0: return "0"
    if (isinstance(value, int)):
        return str(value)
    elif (isinstance(value, float)):
        if value >= 1000:
            r = "{}".format(int(value))
        elif 1000 > value > 10:
            r = "{:.1f}".format(value)
        elif 10 > value >= 1:
            r = "{:.2f}".format(value)
        elif 0.9999 > value > 0.0001:
            r = "{:.4f}".format(value)
        elif 1 > value > 0.9999:
            r = "1"
        else:
            r = "{:.1e}".format(value)
        return r


def multiple_correction(dic):
    """
    dic[ty][r][q] = p
    """
    for ty in dic.keys():
        all_p = []
        rn = len(dic[ty].keys())
        qn = len(dic[ty].values()[0].keys())
        cue = {}
        i = 0
        if rn == 1 and qn == 1: return
        # get all p values from the dictionary
        for r in dic[ty].keys():
            for q in dic[ty][r].keys():

                if isinstance(dic[ty][r][q], str):
                    pass
                else:
                    all_p.append(dic[ty][r][q])
                    cue[ty + r + q] = i
                    i = i + 1
        # correction
        reject, pvals_corrected = multiple_test_correction(all_p, alpha=0.05, method='indep')
        # modify all p values
        for ir, r in enumerate(dic[ty].keys()):
            for iq, q in enumerate(dic[ty][r].keys()):
                try:
                    dic[ty][r][q] = pvals_corrected[cue[ty + r + q]]
                except:
                    pass


def compute_coverage(input):
    """
    bed, bam, rs, bs, ss, center, heatmap, logt, s, g, c, d
    """
    ts = time.time()
    cov = CoverageSet(input[0].name + ".", input[0])
    if ".bigWig" in input[1] or ".bw" in input[1]:
        cov.coverage_from_bigwig(bigwig_file=input[1], stepsize=input[4])
    else:
        cov.coverage_from_bam(bam_file=input[1], extension_size=input[2], binsize=input[3], stepsize=input[4])
        cov.normRPM()
    # When bothends, consider the fliping end
    if input[5] == 'bothends':
        flap = CoverageSet("for flap", input[0])
        flap.coverage_from_bam(input[1], extension_size=input[2], binsize=input[3], stepsize=input[4])
        ffcoverage = numpy.fliplr(flap.coverage)
        cov.coverage = numpy.concatenate((cov.coverage, ffcoverage), axis=0)
    # Averaging the coverage of all regions of each bed file
    if input[6]:
        if input[7]:
            result = numpy.log10(numpy.vstack(cov.coverage))  # Store the array into data list
        else:
            result = numpy.vstack(cov.coverage)  # Store the array into data list
    else:
        # print(cov.coverage)
        for i, car in enumerate(cov.coverage):
            car = numpy.delete(car, [0, 1])
            if i == 0:
                avearr = np.array(car)
                lenr = car.shape[0]
            elif car.shape[0] == lenr:
                avearr = numpy.vstack((avearr, car))
            else:
                pass
        # avearr = numpy.array(cov.coverage)
        # print(avearr)
        # print(avearr.shape)
        avearr = numpy.average(avearr, axis=0)
        # numpy.transpose(avearr)
        result = [input[8], input[9], input[10], input[11], avearr]  # Store the array into data list
    te = time.time()
    print("\tComputing " + os.path.basename(input[1]) + " . " + input[0].name + "\t\t" + str(
        datetime.timedelta(seconds=round(te - ts))))
    return result


def mp_count_intersets(inps):
    # [ com, self.rlen[ty][r.name], self.mode_count, threshold ]
    random_r, random_q = inps[0].random_split(size=inps[1])
    d = random_r.intersect_count(random_q, mode_count=inps[2], threshold=inps[3])
    return d


def mp_count_intersect(inputs):
    # q, nalist, mode_count, qlen_dict, threshold, counts, frequency, self_frequency, ty, r
    q = inputs[0]
    nalist = inputs[1]
    mode_count = inputs[2]
    # qlen_dict = inputs[3]
    threshold = inputs[4]
    # counts = inputs[5]
    frequency = inputs[6]
    # self_frequency = inputs[7]
    ty = inputs[8]
    r = inputs[9]

    output = [q.name]

    if q.total_coverage() == 0 and len(q) > 0:
        output.append(q.name)

    else:
        output.append(None)

        if mode_count == "bp":
            qlen = q.total_coverage()
        elif mode_count == "count":
            qlen = len(q)

        output.append(qlen)
        # Define different mode of intersection and count here
        print(".", end="")
        sys.stdout.flush()
        c = r.intersect_count(q, mode_count=mode_count, threshold=threshold)

        output.append(c)
        # counts[ty][r.name][q.name] = c
        if frequency:
            output.append(c[2])
            # try: self_frequency[ty][q.name].append(c[2])
            # except:
            #    self_frequency[ty][q.name] = [c[2]]

    return output


def get_url(url, filename):
    # file_name = url.split('/')[-1]
    u = urllib2.urlopen(url)
    print("** Loading " + url, end="")
    f = open(filename, 'wb')
    print(u.read(), file=f)
    f.close()
    print("\t... Done")


def load_dump(path, filename):
    print("\tLoading from file: " + filename)
    file = open(os.path.join(path, filename), 'r')
    object = pickle.load(file)
    file.close()
    return object


def dump(object, path, filename):
    print("\tDump to file: " + filename)
    file = open(os.path.join(path, filename), 'wb')
    pickle.dump(object, file)
    file.close()


def read_gtf(gd, organism):
    try:
        anno = load_dump(gd.get_annotation_dump_dir(), "gtf.dump")
        print("ee")
    except:
        anno = AnnotationSet(organism)
        dump(anno, gd.get_annotation_dump_dir(), "gtf.dump")
        print("qq")
    return anno


def annotation_dump(organism):
    print("\nLoading genetic annotation data...\n")
    gd = GenomeData(organism)
    beds = []
    # TSS
    try:
        tss = load_dump(gd.get_annotation_dump_dir(), "tss.dump")
    except:
        anno = read_gtf(gd, organism)
        tss = anno.get_tss()
        dump(tss, gd.get_annotation_dump_dir(), "tss.dump")
    beds.append(tss)
    print("\tTSS: " + str(len(tss)))
    # TTS
    try:
        tts = load_dump(gd.get_annotation_dump_dir(), "tts.dump")
    except:
        anno = read_gtf(gd, organism)
        tts = anno.get_tts()
        dump(tts, gd.get_annotation_dump_dir(), "tts.dump")
    beds.append(tts)
    print("\tTTS: " + str(len(tts)))

    # exon
    try:
        exon = load_dump(gd.get_annotation_dump_dir(), "exon.dump")
    except:
        anno = read_gtf(gd, organism)
        exon = anno.get_exons(start_site=False, end_site=False)
        dump(exon, gd.get_annotation_dump_dir(), "exon.dump")
    print("\texon: " + str(len(exon)))
    # exons
    try:
        exons = load_dump(gd.get_annotation_dump_dir(), "exons.dump")
    except:
        anno = read_gtf(gd, organism)
        exons = anno.get_exons(start_site=True, end_site=False)
        dump(exons, gd.get_annotation_dump_dir(), "exons.dump")
    beds.append(exons)
    print("\texons: " + str(len(exons)))
    # exone
    try:
        exone = load_dump(gd.get_annotation_dump_dir(), "exone.dump")
    except:
        anno = read_gtf(gd, organism)
        exone = anno.get_exons(start_site=False, end_site=True)
        dump(exone, gd.get_annotation_dump_dir(), "exone.dump")
    beds.append(exone)
    print("\texone: " + str(len(exone)))

    # bednames = ["TSS", "TTS", "Exon start site", "Exon end site", "Intron start site", "Intron end site"]
    bednames = ["TSS", "TTS", "Exon start site", "Exon end site"]

    annotation = bednames

    return beds, bednames, annotation



def output(f, directory, folder, filename, extra=None, pdf=False, show=None):
    """Output the file in the defined folder """
    pd = os.path.normpath(os.path.join(dir, directory, folder))
    try:
        os.stat(os.path.dirname(pd))
    except:
        os.mkdir(os.path.dirname(pd))
    try:
        os.stat(pd)
    except:
        os.mkdir(pd)

        # Saving
    if not extra:
        f.savefig(os.path.join(pd, filename), facecolor='w', edgecolor='w',
                  bbox_inches='tight', dpi=400)
    else:
        f.savefig(os.path.join(pd, filename), facecolor='w', edgecolor='w',
                  bbox_extra_artists=(extra), bbox_inches='tight', dpi=400)

    if pdf:
        try:
            pp = PdfPages(os.path.join(pd, filename) + '.pdf')
            pp.savefig(f, bbox_extra_artists=(extra), bbox_inches='tight')
            pp.close()
        except:
            print("ERROR: Problem in PDF conversion. Skipped.")
    if show:
        plt.show()


def output_parameters(parameter, directory, folder, filename):
    pd = os.path.join(dir, directory, folder)
    try:
        os.stat(os.path.dirname(pd))
    except:
        os.mkdir(os.path.dirname(pd))
    try:
        os.stat(pd)
    except:
        os.mkdir(pd)
    if parameter:
        with open(os.path.join(pd, "parameters.txt"), 'w') as f:
            for s in parameter:
                print(s, file=f)


def copy_em(em, directory, folder, filename="experimental_matrix.txt"):
    copyfile(em, os.path.join(dir, directory, folder, filename))


def list_all_index(path):
    """Creat an 'index.html' in the defined directory """
    dirname = os.path.basename(path)
    parentdir = os.path.basename(os.path.dirname(path))

    # link_d = {"List":"index.html"}
    link_d = {}
    ####
    for root, dirnames, filenames in os.walk(os.path.dirname(path)):
        for filename in fnmatch.filter(filenames, 'index.html'):
            if root.split('/')[-2] == parentdir:
                link_d[root.split('/')[-1]] = "../" + root.split('/')[-1] + "/index.html"
    link_d = OrderedDict(sorted(link_d.items(), key=lambda (key, value): key))

    ###

    html = Html(name="Directory: " + dirname, links_dict=link_d,
                fig_dir=os.path.join(path, "style"), fig_rpath="./style", RGT_header=False, other_logo="viz")
    header_list = ["No.", "Experiments"]
    html.add_heading("All experiments in: " + dirname + "/")
    data_table = []
    type_list = 'ssss'
    col_size_list = [10, 10, 10]
    c = 0
    for root, dirnames, filenames in os.walk(path):
        # roots = root.split('/')
        for filename in fnmatch.filter(filenames, '*.html'):
            if filename == 'index.html' and root.split('/')[-1] != dirname:
                # print(root)
                c += 1
                data_table.append([str(c),
                                   '<a href="' + os.path.join(root.split('/')[-1], filename) + '"><font size="4">' +
                                   root.split('/')[-1] + "</a>"])
                # print(link_d[roots[-1]])
    html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=50, cell_align="left", sortable=True)
    html.add_fixed_rank_sortable()
    html.write(os.path.join(path, "index.html"))


def check_dir(path):
    """Check the availability of the given directory and creat it"""
    try:
        os.stat(path)
    except:
        os.mkdir(path)

class NoDaemonProcess(multiprocessing.Process):
    # make 'daemon' attribute always return False
    def _get_daemon(self):
        return False

    def _set_daemon(self, value):
        pass

    daemon = property(_get_daemon, _set_daemon)


# We sub-class multiprocessing.pool.Pool instead of multiprocessing.Pool
# because the latter is only a wrapper function, not a proper class.
class MyPool(multiprocessing.pool.Pool):
    Process = NoDaemonProcess


# Python Libraries
from __future__ import print_function
import os
import copy
# Local Libraries

# Distal Libraries
from Util import SequenceType

####################################################################################
####################################################################################
"""
Sequence
===================
Sequence describes the sequence with ATCG as alphabets as well as its types.

"""
class Sequence():

    def __init__(self, seq, strand, name=None):
        """*Keyword arguments:*

            - seq -- The sequence composef of A, T, C, G, or U.
            - strand -- Orientation of the sequence, '+' or '-'.
        """
        self.name = name # Extra information about the sequence
        self.seq = seq.upper() # Convert all alphabets into upper case
        self.strand = strand
        
    def __len__(self):
        """Return the length of the sequence."""
        return len(self.seq)

    def __str__(self):
        return ":".join([str(x) for x in [self.name, self.seq] if x])
        
    def dna_to_rna(self):
        """Convert the sequence from DNA sequence to RNA sequence."""
        self.seq = self.seq.replace("T","U")
        
    def rna_to_dna(self):
        """Convert the sequence from RNA sequence to DNA sequence."""
        self.seq = self.seq.replace("U", "T")
        
    def GC_content(self):
        """Return the ratio of GC content in this sequence."""
        gc = self.seq.count("G") + self.seq.count("C")
        return gc/float(len(self))

    def methylated_sequence(self,CpG):
#ALPHABET "DNA with covalent modifications" DNA-LIKE

# Core symbols
# A "Adenine" 8510A8 ~ T "Thymine" A89610
# C "Cytosine" A50026 ~ G "Guanine" 313695
# m "5-Methylcytosine" D73027 ~ 1 "Guanine:5-Methylcytosine" 4575B4
        aux=list(self.seq)
        for pos in CpG:
          if self.seq[pos]=="C":
            aux[pos]="m"
          elif self.seq[pos]=="G":
            aux[pos]="1"
          else:
            raise Exception("Position "+str(pos)+"-"+self.seq[pos]+" from "+self.seq+" can not be methylated")         
        self.seq="".join(aux)
        
    def complement(self):
        """Return another Sequence which is the complement to original sequence."""
        if self.strand == "RNA":
            print("No complement strand for RNA "+self.name)
            return
        elif self.strand == "+": strand = "-"
        else: strand = "-"
        
        seq = copy.deepcopy(self.seq)
        seq.replace("A","G")
        seq.replace("G","A")
        seq.replace("C","T")
        seq.replace("T","C")
        
        s = Sequence(seq, name=self.name+"_complement",  strand=strand)
        return s

####################################################################################
####################################################################################

"""
SequenceSet
===================
SequenceSet represents the collection of sequences and their functions.

"""

class SequenceSet:           

    def __init__(self, name, seq_type):
        """*Keyword arguments:*

            - name -- The name of this collection of sequences.
            - seq_type -- DNA or RNA.
        """
        self.name = name
        self.sequences = []
        self.seq_type = seq_type
        
    def __len__(self):
        return len(self.sequences)
        
    def __iter__(self):
        return iter(self.sequences)

    def __getitem__(self, key):
        return self.sequences[key]

    def add(self, seq):
        """Add one more sequence into the object.

        *Keyword arguments:*

            - seq -- A Sequence object.
        """
        self.sequences.append(seq)

    def read_fasta(self, fasta_file):
        """Read all the sequences in the given fasta file.

        *Keyword arguments:*
            -fasta_file -- The path to the FASTA file.
        """
        pre_seq = False
        with open(fasta_file) as f:
            for line in f:
                line = line.strip()
                if not line: pass
                elif line[0] == ">":
                    if pre_seq:
                        self.sequences.append(Sequence(seq=seq, strand=strand, name=info))
                        pre_seq = False
                    info = line.split()[0][1:]
                    seq = ""
                    try: strand = line[line.index("strand")+7]
                    except: strand = "+"
                else:
                    try: seq = seq + line
                    except: seq = line
                    pre_seq = True
            self.sequences.append(Sequence(seq=seq, strand=strand, name=info))

    def read_genomic_set(self, genomic_set, genome_file_dir):
        """Read the sequences defined by a given genomic set.s
        *Keyword arguments:*

            - genomic_set - genomic set with regions to obtain the fasta file
            - genome_file_dir -- A directory which contains the FASTA files for each chromosome.
        """

        bed=genomic_set
        # Parse each chromosome and fetch the defined region in this chromosome
        chroms = list(set(bed.get_chrom()))

        chro_files = [x.split(".")[0] for x in os.listdir(genome_file_dir)]

        for ch in chroms:
            if ch not in chro_files: print(" *** There is no genome FASTA file for: "+ch)

            # Read genome in FASTA according to the given chromosome
            ch_seq = SequenceSet(name=ch, seq_type=SequenceType.DNA)
            try: 
                ch_seq.read_fasta(os.path.join(genome_file_dir, ch+".fa"))
            except:
                continue
            
            # Regions in given chromosome
            beds = bed.any_chrom(chrom=ch)

            for s in beds:
                seq = ch_seq[0].seq[s.initial:s.final]
                try: strand = s.strand
                except: strand = "+"
                self.sequences.append(Sequence(seq=seq, name=s.__repr__(), strand=strand))



    def read_bed(self, bedfile, genome_file_dir):
        """Read the sequences defined by BED file on the given genomce.

        *Keyword arguments:*

            - bedfile -- The path to the BED file which defines the regions.
            - genome_file_dir -- A directory which contains the FASTA files for each chromosome.
        """

        # Read BED into GenomicRegionSet
        from rgt.GenomicRegionSet import GenomicRegionSet
        bed = GenomicRegionSet(os.path.basename(bedfile))
        bed.read_bed(bedfile)
        self.read_genomic_set(bed, genome_file_dir)
        


    def total_len(self):
        """Retrun the total length of the given SequenceSet."""
        tol = 0
        for s in self:
            tol += len(s)
        return tol
 



import sys
import copy
import itertools
from collections import OrderedDict
from rgt.Util import OverlapType
from rgt.GenomicRegionSet import GenomicRegionSet
from rgt.ExperimentalMatrix import ExperimentalMatrix



###########################################################################################
#                    Combinatorial test
###########################################################################################

class Combinatorial:
    def __init__(self, em_path, query):
        self.EM = ExperimentalMatrix()
        self.EM.read(em_path)
        self.regions = self.EM.get_regionsets()
        self.regionnames = self.EM.get_regionsnames()

    def group_refque(self, groupby):
        self.grouped_regions = OrderedDict()  # Store all bed names according to their types
        if groupby:
            for rs in self.regions:
                ty = self.EM.get_type(rs.name, groupby)
                try:
                    self.grouped_regions[ty].append(rs)
                except:
                    self.grouped_regions[ty] = [rs]
        else:
            print("** Please define grouping column '-g'")
            sys.exit(1)

    def posi2region(self, regions, p):
        all = range(len(regions))
        new_r = GenomicRegionSet(name="")
        for r in p:
            new_r.combine(regions[r])
        return new_r

    def posi2set(self, regions, p):
        all = range(len(regions))
        inter_r = copy.deepcopy(regions[p[0]])

        for i in all:
            # print("inter_r: "+inter_r.name)
            if i in p[1:]:
                inter_r = inter_r.intersect(regions[i], mode=OverlapType.OVERLAP)
            elif i == p[0]:
                pass
            else:
                inter_r = inter_r.subtract(regions[i], whole_region=False)
        # print("inter_r: "+inter_r.name)
        return inter_r

    def combinatorial(self, background=None):
        def p2sign(plist, length):
            output = ["-"] * length
            for p in plist:
                output[p] = "+"
            return output

        new_refsp = OrderedDict()
        new_refs = OrderedDict()
        ref_names = []
        self.comb_ref_infor = {}

        for ty in self.groupedreference.keys():
            n = len(self.groupedreference[ty])
            new_refs[ty] = []
            new_refsp[ty] = []

            for i in range(1, n):
                new_refsp[ty].append(itertools.combinations(range(n), i))
            for posi in new_refsp[ty]:
                posi = [list(i) for i in posi]

                for p in posi:
                    # print("   " + str(p))
                    pr = self.posi2set(self.groupedreference[ty], p)
                    new_refs[ty].append(pr)
                    ref_names.append(pr.name)
                    self.comb_ref_infor[pr.name] = p2sign(p, n)
            all_int = self.posi2set(self.groupedreference[ty], range(n))
            new_refs[ty].append(all_int)
            ref_names.append(all_int.name)
            self.comb_ref_infor[all_int.name] = p2sign(range(n), n)
            """
            # Background
            unions = GenomicRegionSet(name="")
            for r in self.groupedreference[ty]:
                unions.combine(r)
            unions.name = " + ".join([r.name for r in self.groupedreference[ty]])

            nonset = self.backgroung.subtract(unions)
            nonset.name = "!("+"+".join([r.name for r in self.groupedreference[ty]]) + ")"
            new_refs[ty].append(nonset)
            ref_names.append(nonset.name)
            """
        # self.comb_reference = new_refs
        self.groupedreference = copy.deepcopy(new_refs)
        self.orig_refs = copy.deepcopy(self.referencenames)
        self.referencenames = list(set(ref_names))

    def combine_regions(self, background=None):
        new_refsp = OrderedDict()
        new_refs = OrderedDict()
        ref_names = []

        for ty in self.groupedreference.keys():
            n = len(self.groupedreference[ty])
            new_refs[ty] = []
            new_refsp[ty] = []
            for i in range(1, n):
                new_refsp[ty].append(itertools.combinations(range(n), i))
            for posi in new_refsp[ty]:
                posi = [list(i) for i in posi]
                for p in posi:
                    print("   " + str(p))
                    pr = self.posi2region(self.groupedreference[ty], p)
                    new_refs[ty].append(pr)
                    ref_names.append(pr.name)


"""
THOR detects differential peaks in multiple ChIP-seq profiles between
two distinct biological conditions.

Copyright (C) 2014-2016 Manuel Allhof (allhoff@aices.rwth-aachen.de)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

@author: Manuel Allhoff
"""

from __future__ import print_function
import sys
import gc
from random import sample
import numpy as np
from normalize import get_normalization_factor
from DualCoverageSet import DualCoverageSet
from norm_genelevel import norm_gene_level
from rgt.CoverageSet import CoverageSet, get_gc_context

EPSILON = 1**-320
ROUND_PRECISION = 3
DEBUG = None
VERBOSE = None


class MultiCoverageSet(DualCoverageSet):
    def _help_init(self, path_bamfiles, exts, rmdup, binsize, stepsize, path_inputs, exts_inputs, dim, regions, norm_regionset, strand_cov):
        """Return self.covs and self.inputs as CoverageSet"""
        self.exts = exts
        self.covs = [CoverageSet('file' + str(i), regions) for i in range(dim)]
        for i, c in enumerate(self.covs):
            c.coverage_from_bam(bam_file=path_bamfiles[i], extension_size=exts[i], rmdup=rmdup, binsize=binsize,\
                                stepsize=stepsize, get_strand_info = strand_cov)
        self.covs_avg = [CoverageSet('cov_avg'  + str(i) , regions) for i in range(2)]
        if path_inputs:
            self.inputs = [CoverageSet('input' + str(i), regions) for i in range(len(path_inputs))]
            for i, c in enumerate(self.inputs):
                c.coverage_from_bam(bam_file=path_inputs[i], extension_size=exts_inputs[i], rmdup=rmdup, binsize=binsize,\
                                stepsize=stepsize, get_strand_info = strand_cov)
            self.input_avg = [CoverageSet('input_avg'  + str(i), regions) for i in range(2)]
        else:
            self.inputs = []
            
        if norm_regionset:
            self.norm_regions = [CoverageSet('norm_region' + str(i), norm_regionset) for i in range(dim)]
            for i, c in enumerate(self.norm_regions):
                c.coverage_from_bam(bam_file=path_bamfiles[i], extension_size=exts[i], rmdup=rmdup, binsize=binsize,\
                                    stepsize=stepsize, get_strand_info = strand_cov)
            self.input_avg = [CoverageSet('input_avg'  + str(i), regions) for i in range(2)]
        else:
            self.norm_regions = None
    
    def _get_covs(self, DCS, i):
        """For a multivariant Coverageset, return coverage cov1 and cov2 at position i"""
        cov1 = int(np.mean(DCS.overall_coverage[0][:,DCS.indices_of_interest[i]]))
        cov2 = int(np.mean(DCS.overall_coverage[1][:,DCS.indices_of_interest[i]]))
    
        return cov1, cov2
    
    def _compute_gc_content(self, no_gc_content, path_inputs, stepsize, binsize, genome_path, name, chrom_sizes, chrom_sizes_dict):
        """Compute GC-content"""
        if not no_gc_content and path_inputs and self.gc_content_cov is None:
            print("Compute GC-content", file=sys.stderr)
            for i, cov in enumerate(self.covs):
                inputfile = self.inputs[i] #1 to 1 mapping between input and cov
                rep = i if i < self.dim_1 else i-self.dim_1
                sig = 1 if i < self.dim_1 else 2
                self.gc_content_cov, self.avg_gc_content, self.gc_hist = get_gc_context(stepsize, binsize, genome_path, inputfile.coverage, chrom_sizes_dict)
                self._norm_gc_content(cov.coverage, self.gc_content_cov, self.avg_gc_content)
                self._norm_gc_content(inputfile.coverage, self.gc_content_cov, self.avg_gc_content)
            
                #if VERBOSE:
                #    self.print_gc_hist(name + '-s%s-rep%s-' %(sig, rep), gc_hist)
                #    cov.write_bigwig(name + '-s%s-rep%s-gc.bw' %(sig, rep), chrom_sizes)
        
    
    def _output_input_bw(self, name, chrom_sizes, save_wig):
        for i in range(len(self.covs)):
            rep = i if i < self.dim_1 else i-self.dim_1
            sig = 1 if i < self.dim_1 else 2
            if self.inputs:
                self.inputs[i].write_bigwig(name + '-' + str(self.counter) + '-input-s%s-rep%s.bw' %(sig, rep), chrom_sizes, save_wig=save_wig, end=self.end)
    
    def _output_bw(self, name, chrom_sizes, save_wig, save_input):
        """Output bigwig files"""
        for i in range(len(self.covs)):
            rep = i if i < self.dim_1 else i-self.dim_1
            sig = 1 if i < self.dim_1 else 2
            
            self.covs[i].write_bigwig(name + '-' + str(self.counter) + '-s%s-rep%s.bw' %(sig, rep), chrom_sizes, save_wig=save_wig, end=self.end)
        
        #ra = [self.covs_avg, self.input_avg] if self.inputs else [self.covs_avg]
        #for k, d in enumerate(ra):
        #    g = self.covs if k == 0 else self.inputs
        #    for j in range(2):
        #        d[j] = deepcopy(g[0]) if j == 0 else deepcopy(g[self.dim_1])
        #        r = range(1, self.dim_1) if j == 0 else range(self.dim_1 + 1, self.dim_1 + self.dim_2)
        #        f = 1./self.dim_1 if j == 0 else 1./self.dim_2
        #        for i in r:
        #            d[j].add(g[i])
        #        d[j].scale(f)
        #        n = name + '-s%s.bw' %(j+1) if k == 0 else name + '-s%s-input.bw' %(j+1)
        #        d[j].write_bigwig(n, chrom_sizes, save_wig=save_wig, end=self.end)
        
        self.covs_avg = None
        self.input_avg = None
        if self.inputs:
            for i in range(len(self.covs)):
                self.inputs[i] = None #last time that we need this information, delete it
        gc.collect()
        
    
    def _help_get_data(self, i, type):
        if type != 'normregion':
            for j in range(len(self.covs[i].genomicRegions)):
                if type == 'cov':
                    yield self.covs[i].coverage[j]
                elif type == 'strand':
                    yield self.covs[i].cov_strand_all[j]
        elif type == 'normregion':
            for j in range(len(self.norm_regions[i].genomicRegions)):
                yield self.norm_regions[i].coverage[j]
    
    def _help_init_overall_coverage(self, cov_strand=True):
        """Convert coverage data (and optionally strand data) to matrix list"""
        
        tmp = [[], []]
        tmp2 = [[[], []], [[], []]]
        
        for k in range(2):
            it = range(self.dim_1) if k == 0 else range(self.dim_1, self.dim_1 + self.dim_2)
            for i in it:
                if cov_strand:
                    tmp_el = reduce(lambda x,y: np.concatenate((x,y)), self._help_get_data(i, 'cov'))
                    tmp[k].append(tmp_el)
                 
                    tmp_el = map(lambda x: (x[0], x[1]), reduce(lambda x,y: np.concatenate((x,y)), self._help_get_data(i, 'strand')))
                    tmp2[k][0].append(map(lambda x: x[0], tmp_el))
                    tmp2[k][1].append(map(lambda x: x[1], tmp_el))
                else:
                    tmp_el = reduce(lambda x,y: np.concatenate((x,y)), self._help_get_data(i, 'normregion'))
                    tmp[k].append(tmp_el)

        if cov_strand:
            #1. or 2. signal -> pos/neg strand -> matrix with rep x bins
            overall_coverage_strand = [[np.matrix(tmp2[0][0]), np.matrix(tmp2[0][1])], [np.matrix(tmp2[1][0]), np.matrix(tmp2[0][1])]]
            #list of matrices: #replicates (row) x #bins (columns)
            overall_coverage = [np.matrix(tmp[0]), np.matrix(tmp[1])]
         
            return overall_coverage, overall_coverage_strand
        else:
            return [np.matrix(tmp[0]), np.matrix(tmp[1])]
    
    def count_positive_signal(self):
        return np.sum([self.covs[i].coverage for i in range(self.dim_1 + self.dim_2)])
    
    def __init__(self, name, dims, regions, genome_path, binsize, stepsize, chrom_sizes, norm_regionset, \
                 verbose, debug, no_gc_content, rmdup, path_bamfiles, exts, path_inputs, exts_inputs, \
                 factors_inputs, chrom_sizes_dict, scaling_factors_ip, save_wig, strand_cov, housekeeping_genes,\
                 tracker, end, counter, gc_content_cov=None, avg_gc_content=None, gc_hist=None, output_bw=True,\
                 folder_report=None, report=None, save_input=False, m_threshold=80, a_threshold=95):
        """Compute CoverageSets, GC-content and normalize input-DNA and IP-channel"""
        self.genomicRegions = regions
        self.binsize = binsize
        self.stepsize = stepsize
        self.name = name
        self.chrom_sizes_dict = chrom_sizes_dict
        self.dim_1, self.dim_2 = dims
        self.exts = exts
        self.exts_inputs = exts_inputs
        self.gc_content_cov = gc_content_cov
        self.avg_gc_content = avg_gc_content
        self.gc_hist = gc_hist
        self.scaling_factors_ip = scaling_factors_ip
        self.factors_inputs = factors_inputs
        self.end = end
        self.counter = counter
        self.no_data = False
        self.FOLDER_REPORT = folder_report
        global DEBUG, VERBOSE
        DEBUG = debug
        VERBOSE = verbose
        
        #make data nice
        self._help_init(path_bamfiles, exts, rmdup, binsize, stepsize, path_inputs, exts_inputs, sum(dims), regions, norm_regionset, strand_cov = strand_cov)
        if self.count_positive_signal() < 1:
            self.no_data = True
            return None
        self._compute_gc_content(no_gc_content, path_inputs, stepsize, binsize, genome_path, name, chrom_sizes, chrom_sizes_dict)
        self._normalization_by_input(path_bamfiles, path_inputs, name, factors_inputs, save_input)
        if save_input:
            self._output_input_bw(name, chrom_sizes, save_wig) 
            
        self.overall_coverage, self.overall_coverage_strand = self._help_init_overall_coverage(cov_strand=True)
        
        self._normalization_by_signal(name, scaling_factors_ip, path_bamfiles, housekeeping_genes, tracker, norm_regionset, report,
                                      m_threshold, a_threshold)
        
        if output_bw:
            self._output_bw(name, chrom_sizes, save_wig, save_input) 
        
        self.scores = np.zeros(len(self.overall_coverage[0]))
        self.indices_of_interest = []
    
    def get_max_colsum(self):
        """Sum over all columns and add maximum"""
        return self.overall_coverage[0].sum(axis=0).max() + self.overall_coverage[1].sum(axis=0).max()
    
    def output_overall_coverage(self, path):
        for j in range(2):
            f = open(path + str(j), 'w')
            for i in range(self.overall_coverage[j].shape[1]):
                print(self.overall_coverage[j][:,i].T, file=f)
    
    def _normalization_by_input(self, path_bamfiles, path_inputs, name, factors_inputs, save_input):
        """Normalize input-DNA. Use predefined factors or follow Diaz et al, 2012"""
        
        if VERBOSE:
            print("Normalize input-DNA", file=sys.stderr)
        
        if factors_inputs:
            if VERBOSE:
                print("Use with predefined factors", file=sys.stderr)
            for i in range(len(path_bamfiles)):
                self.inputs[i].scale(factors_inputs[i])
                self.covs[i].subtract(self.inputs[i])
        elif path_inputs:
            factors_inputs = []
            print("Compute factors", file=sys.stderr)
            for i in range(len(path_bamfiles)):
                rep = i if i < self.dim_1 else i-self.dim_1
                sig = 0 if i < self.dim_1 else 1
                j = 0 if i < self.dim_1 else 1
                _, n = get_normalization_factor(path_bamfiles[i], path_inputs[i], step_width=1000, zero_counts=0, \
                                                filename=name + '-norm' + str(i), debug=DEBUG, chrom_sizes_dict=self.chrom_sizes_dict, two_sample=False, stop=True)
                if n is not None:
                    print("Normalize input of Signal %s, Rep %s with factor %s"\
                           %(sig, rep, round(n, ROUND_PRECISION)) , file=sys.stderr)
                    self.inputs[i].scale(n)
                    
                    self.covs[i].subtract(self.inputs[i])
                    factors_inputs.append(n)
        
        self.factors_inputs = factors_inputs
        
                    
    def _trim4TMM(self, m_values, a_values, m_threshold=80, a_threshold=95):
        """q=20 or q=5"""
        assert len(m_values) == len(a_values)
        
        mask = np.asarray([not x for x in np.isinf(m_values) + np.isinf(a_values)])
        
        m_values = m_values[mask]
        a_values = a_values[mask]
        
        perc_m_l = np.percentile(m_values, 100-m_threshold)
        perc_m_h = np.percentile(m_values, m_threshold)
        perc_a_l = np.percentile(a_values, 100-a_threshold)
        perc_a_h = np.percentile(a_values, a_threshold)
        
        try:
            res = filter(lambda x: not(x[0]>perc_m_h or x[0]<perc_m_l),\
                     filter(lambda x: not(x[1]>perc_a_h or x[1]<perc_a_l), zip(list(m_values.squeeze()),list(a_values.squeeze()))))
        except:
            print('something wrong %s %s' %(len(m_values), len(a_values)), file=sys.stderr)
            return np.asarray(m_values), np.asarray(a_values)
        
        if res:
            return np.asarray(map(lambda x: x[0], res)), np.asarray(map(lambda x: x[1], res))
        else:
            print('TMM normalization: nothing trimmed...', file=sys.stderr)
            return np.asarray(m_values), np.asarray(a_values)
    
    def _norm_TMM(self, overall_coverage, m_threshold, a_threshold):
        """Normalize with TMM approach, based on PePr"""
        scaling_factors_ip = []
        for j, cond_max in enumerate([self.dim_1, self.dim_2]):
            for i in range(cond_max): #normalize all replicates
                ref = np.asarray(np.sum(overall_coverage[0], axis=0) + np.sum(overall_coverage[1], axis=0), dtype='float')/ (self.dim_1 + self.dim_2)
                
                mask_ref = ref > 0
                ref = ref[mask_ref]
                data_rep = np.asarray(overall_coverage[j][i,:])[mask_ref]
                tmp = zip(data_rep, ref, data_rep + ref)
                tmp.sort(key = lambda x: x[2], reverse=True)
                tmp = tmp[:min(len(tmp), 10000)]
                
                data_rep = np.asarray(map(lambda x: x[0], tmp))
                ref = np.asarray(map(lambda x: x[1], tmp))
                assert len(data_rep) == len(ref)
                m = data_rep > 0
                data_rep = data_rep[m]
                ref = ref[m]
                
                m_values = np.log(ref / data_rep)
                a_values = 0.5 * np.log(data_rep * ref)
                try:
                    m_values, a_values = self._trim4TMM(m_values, a_values, m_threshold, a_threshold)
                    f = 2 ** (np.sum(m_values * a_values) / np.sum(a_values))
                    scaling_factors_ip.append(f)
                except:
                    print('TMM normalization not successfully performed, do not normalize data', file=sys.stderr)
                    scaling_factors_ip.append(1)
                
        return scaling_factors_ip
    
    def _normalization_by_signal(self, name, scaling_factors_ip, bamfiles, housekeeping_genes, tracker, norm_regionset, report,
                                 m_threshold, a_threshold):
        """Normalize signal"""
        
        if VERBOSE:
            print('Normalize ChIP-seq profiles', file=sys.stderr)
        
        if not scaling_factors_ip and housekeeping_genes:
            print('Use housekeeping gene approach', file=sys.stderr)
            scaling_factors_ip, _ = norm_gene_level(bamfiles, housekeeping_genes, name, verbose=True, folder = self.FOLDER_REPORT, report=report)
        elif not scaling_factors_ip:
            if norm_regionset:
                print('Use TMM approach based on peaks', file=sys.stderr)
                norm_regionset_coverage = self._help_init_overall_coverage(cov_strand=False) #TMM approach based on peaks
                scaling_factors_ip = self._norm_TMM(norm_regionset_coverage,m_threshold, a_threshold)
            else:
                print('Use global TMM approach ', file=sys.stderr)
                scaling_factors_ip = self._norm_TMM(self.overall_coverage, m_threshold, a_threshold) #TMM approach
        
        for i in range(len(scaling_factors_ip)):
            self.covs[i].scale(scaling_factors_ip[i]) 
        
        if scaling_factors_ip:
            for j, cond in enumerate([self.dim_1, self.dim_2]):
                for i in range(cond): #normalize all replicates
                    k = i if j == 0 else i+self.dim_1
                    self.overall_coverage[j][i,:] *= scaling_factors_ip[k]
                    if DEBUG:
                        print('Use scaling factor %s' %round(scaling_factors_ip[k], ROUND_PRECISION), file=sys.stderr)
        
        self.scaling_factors_ip = scaling_factors_ip
        
        
        
                
    def _index2coordinates(self, index):
        """Translate index within coverage array to genomic coordinates."""
        iter = self.genomicRegions.__iter__()
        r = iter.next()
        sum = r.final
        last = 0
        i = 0
        while sum <= index * self.stepsize:
            last += len(self.covs[0].coverage[i])
            try:
                r = iter.next()
            except StopIteration:
                sum += r.final
                i += 1
                break
            sum += r.final
            i += 1
        
        return r.chrom, (index-last) * self.stepsize, \
            min((index-last) * self.stepsize + self.stepsize, r.final)
                              
    def __len__(self):
        """Return number of observations."""
        return len(self.indices_of_interest)
    
    def get_observation(self, mask=np.array([])):
        """Return indices of observations. Do not consider indices contained in <mask> array"""
        mask = np.asarray(mask)
        if not mask.size:
            mask = np.array([True]*self._get_bin_number())
        return np.asarray(np.concatenate((self.overall_coverage[0][:,mask].T, self.overall_coverage[1][:,mask].T), axis=1))
    
    def _compute_score(self):
        """Compute score for each observation (based on Xu et al.)"""
        self.scores = sum([np.squeeze(np.asarray(np.mean(self.overall_coverage[i], axis=0))) / float(np.mean(self.overall_coverage[i])) for i in range(2)])
    
    def _get_bin_number(self):
        """Return number of bins"""
        return self.overall_coverage[0].shape[1]
    
    def compute_putative_region_index(self, l=5):
        """Compute putative differential peak regions as follows: 
        - score must be > 0, i.e. everthing
        - overall coverage in library 1 and 2 must be > 3"""
        
        try:
            self._compute_score()
            self.indices_of_interest = np.where(self.scores > 0)[0] #2/(m*n)
            tmp = np.where(np.squeeze(np.asarray(np.mean(self.overall_coverage[0], axis=0))) + np.squeeze(np.asarray(np.mean(self.overall_coverage[1], axis=0))) > 10)[0]
            tmp2 = np.intersect1d(self.indices_of_interest, tmp)
            self.indices_of_interest = tmp2
        except:
            self.indices_of_interest = None
        #print(len(self.indices_of_interest), file=sys.stderr)
        #tmp = set()
        #for i in self.indices_of_interest:
        #    for j in range(max(0, i-l), i+l+1):
        #        tmp.add(j)
        #tmp = list(tmp)
        #tmp.sort()
        #self.indices_of_interest = np.array(tmp)
         
    def write_test_samples(self, name, l):
        f = open(name, 'w')
        
        for el1, el2 in l:
            print(el1, el2, sep='\t', file=f)
        f.close()
    
    def output_training_set(self, name, training_set, s0_v, s1_v, s2_v):
        """Output debug info for training_set computation."""
        f=open(name + '-trainingset.bed', 'w')
        for l in training_set:
            chrom, s, e = self._index2coordinates(l)
            print(chrom, s, e, sep ='\t', file=f)
        f.close()
        
        self.write_test_samples(name + '-s0', s0_v)
        self.write_test_samples(name + '-s1', s1_v)
        self.write_test_samples(name + '-s2', s2_v)
    
    def get_training_set(self, test, exp_data, name, foldchange, min_t, y=5000, ex=2):
        """Return HMM's training set (max <y> positions). Enlarge each contained bin by <ex>."""
        threshold = foldchange
        diff_cov = int(np.percentile(np.abs(np.squeeze(np.asarray(np.mean(self.overall_coverage[0], axis=0))) - \
                                            np.squeeze(np.asarray(np.mean(self.overall_coverage[1], axis=0)))), min_t))

        if test:
            diff_cov, threshold = 2, 1.5
        
        if DEBUG:  
            print('Training set parameters: threshold: %s, diff_cov: %s' %(threshold, diff_cov), file=sys.stderr)
        
        s0, s1, s2 = [], [], []
        
        #compute training set parameters, re-compute training set if criteria do not hold
        rep=True
        while rep:
            for i in sample(range(len(self.indices_of_interest)), min(y, len(self.indices_of_interest))):
                cov1, cov2 = self._get_covs(exp_data, i)
                
                #apply criteria for initial peak calling
                if (cov1 / max(float(cov2), 1) > threshold and cov1+cov2 > diff_cov/2) or cov1-cov2 > diff_cov:
                    s1.append((self.indices_of_interest[i], cov1, cov2))
                elif (cov1 / max(float(cov2), 1) < 1/threshold and cov1+cov2 > diff_cov/2) or cov2-cov1 > diff_cov:
                    s2.append((self.indices_of_interest[i], cov1, cov2))
                else:
                    s0.append((self.indices_of_interest[i], cov1, cov2))
            
                if len(s0) > y and len(s1) > y and len(s2) > y:
                    break
            
            if diff_cov == 1 and threshold == 1.1:
                print("No differential peaks detected", file=sys.stderr)
                sys.exit()
            
            if len(s1) < 100/2 and len(s2) > 2*100:
                s1 = map(lambda x: (x[0], x[2], x[1]), s2)
            if len(s2) < 100/2 and len(s1) > 2*100:
                s2 = map(lambda x: (x[0], x[2], x[1]), s1)
            
            if len(s1) < 100 or len(s2) < 100:
                diff_cov -= 15
                threshold -= 0.1
                diff_cov = max(diff_cov, 1)
                threshold = max(threshold, 1.1)
            else:
                rep = False
        
        if DEBUG:       
            print('Final training set parameters: threshold: %s, diff_cov: %s' %(threshold, diff_cov), file=sys.stderr)
        
        #optimize training set, extend each bin
        tmp = []
        for i, el in enumerate([s0, s1, s2]):
            el = np.asarray(el)
            if not test:
                el = el[el[:,1] < np.percentile(el[:,1], 90)]
                el = el[el[:,2] < np.percentile(el[:,2], 90)]
            tmp.append(el)
        
        s0 = tmp[0]
        s1 = tmp[1]
        s2 = tmp[2]
        
        l = np.min([len(s1), len(s2), len(s0), y])
        
        s0 = sample(s0, l)
        s1 = sample(s1, l)
        s2 = sample(s2, l)
        
        s0_v = map(lambda x: (x[1], x[2]), s0)
        s1_v = map(lambda x: (x[1], x[2]), s1)
        s2_v = map(lambda x: (x[1], x[2]), s2)
        
        extension_set = set()
        for i, _, _ in s0 + s1 + s2:
            for j in range(max(0, i - ex), i + ex + 1): #extend bins
                extension_set.add(j)
         
        tmp = s0 + s1 + s2
        training_set = map(lambda x: x[0], tmp) + list(extension_set)
         
        training_set = list(training_set)
        training_set.sort()
        
        if DEBUG:
            self.output_training_set(name, training_set, s0_v, s1_v, s2_v)
        
        return training_set, s0_v, s1_v, s2_v
        
        